@@ -2,7 +2,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -42,9 +42,9 @@
 module Control.Lens
   ( module Control.Lens.Action
   , module Control.Lens.At
-  , module Control.Lens.Combinators
   , module Control.Lens.Cons
   , module Control.Lens.Each
+  , module Control.Lens.Empty
   , module Control.Lens.Equality
   , module Control.Lens.Fold
   , module Control.Lens.Getter
@@ -58,7 +58,6 @@ module Control.Lens
   , module Control.Lens.Reified
   , module Control.Lens.Review
   , module Control.Lens.Setter
-  , module Control.Lens.Simple
 #ifndef DISABLE_TEMPLATE_HASKELL
   , module Control.Lens.TH
 #endif
@@ -66,15 +65,14 @@ module Control.Lens
   , module Control.Lens.Tuple
   , module Control.Lens.Type
   , module Control.Lens.Wrapped
-  , module Control.Lens.Zipper
   , module Control.Lens.Zoom
   ) where
 
 import Control.Lens.Action
 import Control.Lens.At
-import Control.Lens.Combinators
 import Control.Lens.Cons
 import Control.Lens.Each
+import Control.Lens.Empty
 import Control.Lens.Equality
 import Control.Lens.Fold
 import Control.Lens.Getter
@@ -88,7 +86,6 @@ import Control.Lens.Prism
 import Control.Lens.Reified
 import Control.Lens.Review
 import Control.Lens.Setter
-import Control.Lens.Simple
 #ifndef DISABLE_TEMPLATE_HASKELL
 import Control.Lens.TH
 #endif
@@ -96,7 +93,8 @@ import Control.Lens.Traversal
 import Control.Lens.Tuple
 import Control.Lens.Type
 import Control.Lens.Wrapped
-import Control.Lens.Zipper
 import Control.Lens.Zoom
 
+#ifdef HLINT
 {-# ANN module "HLint: ignore Use import/export shortcut" #-}
+#endif
