@@ -11,7 +11,7 @@
 ----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Fold
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -23,7 +23,7 @@
 -- @'Data.Foldable.foldMap' :: ('Foldable' t, 'Monoid' m) => (a -> m) -> t a -> m@.
 -- Since we want to be able to work with monomorphic containers, we could
 -- generalize this signature to @forall m. 'Monoid' m => (a -> m) -> s -> m@,
--- and then decorate it with 'Accessor' to obtain
+-- and then decorate it with 'Const' to obtain
 --
 -- @type 'Fold' s a = forall m. 'Monoid' m => 'Getting' m s a@
 --
@@ -54,8 +54,9 @@ module Control.Lens.Fold
   , has, hasn't
 
   -- ** Building Folds
-  , folding
+  , folding, ifolding
   , folded
+  , folded64
   , unfolded
   , iterated
   , filtered
@@ -65,12 +66,13 @@ module Control.Lens.Fold
   , cycled
   , takingWhile
   , droppingWhile
+  , worded, lined
 
   -- ** Folding
   , foldMapOf, foldOf
   , foldrOf, foldlOf
   , toListOf
-  , anyOf, allOf
+  , anyOf, allOf, noneOf
   , andOf, orOf
   , productOf, sumOf
   , traverseOf_, forOf_, sequenceAOf_
@@ -84,6 +86,7 @@ module Control.Lens.Fold
   , maximumOf, minimumOf
   , maximumByOf, minimumByOf
   , findOf
+  , findMOf
   , foldrOf', foldlOf'
   , foldr1Of, foldl1Of
   , foldr1Of', foldl1Of'
@@ -100,12 +103,14 @@ module Control.Lens.Fold
   , ifoldlOf
   , ianyOf
   , iallOf
+  , inoneOf
   , itraverseOf_
   , iforOf_
   , imapMOf_
   , iforMOf_
   , iconcatMapOf
   , ifindOf
+  , ifindMOf
   , ifoldrOf'
   , ifoldlOf'
   , ifoldrMOf
@@ -125,6 +130,12 @@ module Control.Lens.Fold
   , Rightmost
   , Traversed
   , Sequenced
+
+  -- * Fold with Reified Monoid
+  , foldBy
+  , foldByOf
+  , foldMapBy
+  , foldMapByOf
   ) where
 
 import Control.Applicative as Applicative
@@ -140,7 +151,10 @@ import Control.Monad as Monad
 import Control.Monad.Reader
 import Control.Monad.State
 import Data.Foldable as Foldable
+import Data.Functor.Apply
 import Data.Functor.Compose
+import Data.Int (Int64)
+import Data.List (intercalate)
 import Data.Maybe
 import Data.Monoid
 import Data.Profunctor
@@ -151,6 +165,7 @@ import Data.Traversable
 -- $setup
 -- >>> :set -XNoOverloadedStrings
 -- >>> import Control.Lens
+-- >>> import Control.Lens.Extras (is)
 -- >>> import Data.Function
 -- >>> import Data.List.Lens
 -- >>> import Debug.SimpleReflect.Expr
@@ -163,9 +178,11 @@ import Data.Traversable
 -- >>> let g :: Expr -> Expr; g = Debug.SimpleReflect.Vars.g
 -- >>> let timingOut :: NFData a => a -> IO a; timingOut = fmap (fromMaybe (error "timeout")) . timeout (5*10^6) . evaluate . force
 
+#ifdef HLINT
 {-# ANN module "HLint: ignore Eta reduce" #-}
 {-# ANN module "HLint: ignore Use camelCase" #-}
 {-# ANN module "HLint: ignore Use curry" #-}
+#endif
 
 infixl 8 ^.., ^?, ^?!, ^@.., ^@?, ^@?!
 
@@ -183,7 +200,11 @@ folding :: (Foldable f, Contravariant g, Applicative g) => (s -> f a) -> LensLik
 folding sfa agb = coerce . traverse_ agb . sfa
 {-# INLINE folding #-}
 
--- | Obtain a 'Fold' from any 'Foldable'.
+ifolding :: (Foldable f, Indexable i p, Contravariant g, Applicative g) => (s -> f (i, a)) -> Over p g s t a b
+ifolding sfa f = coerce . traverse_ (coerce . uncurry (indexed f)) . sfa
+{-# INLINE ifolding #-}
+
+-- | Obtain a 'Fold' from any 'Foldable' indexed by ordinal position.
 --
 -- >>> Just 3^..folded
 -- [3]
@@ -193,11 +214,20 @@ folding sfa agb = coerce . traverse_ agb . sfa
 --
 -- >>> [(1,2),(3,4)]^..folded.both
 -- [1,2,3,4]
-folded :: Foldable f => Fold (f a) a
-folded f = coerce . getFolding . foldMap (Folding #. f)
+folded :: Foldable f => IndexedFold Int (f a) a
+folded = conjoined folded' (indexing folded')
 {-# INLINE folded #-}
 
--- | 'Fold' by repeating the input forever.
+-- | Obtain a 'Fold' from any 'Foldable' indexed by ordinal position.
+folded64 :: Foldable f => IndexedFold Int64 (f a) a
+folded64 = conjoined folded' (indexing64 folded')
+{-# INLINE folded64 #-}
+
+folded' :: Foldable f => Fold (f a) a
+folded' f = coerce . getFolding . foldMap (Folding #. f)
+{-# INLINE folded' #-}
+
+-- | Form a 'Fold1' by repeating the input forever.
 --
 -- @
 -- 'repeat' ≡ 'toListOf' 'repeated'
@@ -205,8 +235,8 @@ folded f = coerce . getFolding . foldMap (Folding #. f)
 --
 -- >>> timingOut $ 5^..taking 20 repeated
 -- [5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5]
-repeated :: Fold a a
-repeated f a = as where as = f a *> as
+repeated :: Fold1 a a
+repeated f a = as where as = f a .> as
 {-# INLINE repeated #-}
 
 -- | A 'Fold' that replicates its input @n@ times.
@@ -224,12 +254,12 @@ replicated n0 f a = go n0 where
   go n = m *> go (n - 1)
 {-# INLINE replicated #-}
 
--- | Transform a 'Fold' into a 'Fold' that loops over its elements over and over.
+-- | Transform a non-empty 'Fold' into a 'Fold1' that loops over its elements over and over.
 --
 -- >>> timingOut $ [1,2,3]^..taking 7 (cycled traverse)
 -- [1,2,3,1,2,3,1]
-cycled :: (Contravariant f, Applicative f) => LensLike f s t a b -> LensLike f s t a b
-cycled l f a = as where as = l f a *> as
+cycled :: (Contravariant f, Apply f) => LensLike f s t a b -> LensLike f s t a b
+cycled l f a = as where as = l f a .> as
 {-# INLINE cycled #-}
 
 -- | Build a 'Fold' that unfolds its values from a seed.
@@ -247,19 +277,19 @@ unfolded f g b0 = go b0 where
     Nothing      -> noEffect
 {-# INLINE unfolded #-}
 
--- | @x '^.' 'iterated' f@ returns an infinite 'Fold' of repeated applications of @f@ to @x@.
+-- | @x '^.' 'iterated' f@ returns an infinite 'Fold1' of repeated applications of @f@ to @x@.
 --
 -- @
 -- 'toListOf' ('iterated' f) a ≡ 'iterate' f a
 -- @
-iterated :: (a -> a) -> Fold a a
+iterated :: (a -> a) -> Fold1 a a
 iterated f g a0 = go a0 where
-  go a = g a *> go (f a)
+  go a = g a .> go (f a)
 {-# INLINE iterated #-}
 
--- | Obtain a 'Fold' that can be composed with to filter another 'Lens', 'Iso', 'Getter', 'Fold' (or 'Traversal').
+-- | Obtain an 'AffineFold' that can be composed with to filter another 'Lens', 'Iso', 'Getter', 'Fold' (or 'Traversal').
 --
--- Note: This is /not/ a legal 'Traversal', unless you are very careful not to invalidate the predicate on the target.
+-- Note: This is /not/ a legal 'AffineTraversal', unless you are very careful not to invalidate the predicate on the target.
 --
 -- Note: This is also /not/ a legal 'Prism', unless you are very careful not to inject a value that matches the predicate.
 --
@@ -269,13 +299,13 @@ iterated f g a0 = go a0 where
 -- 'Control.Lens.Setter.over' evens 'succ' '.' 'Control.Lens.Setter.over' evens 'succ' '/=' 'Control.Lens.Setter.over' evens ('succ' '.' 'succ')
 -- @
 --
--- So, in order for this to qualify as a legal 'Traversal' you can only use it for actions that preserve the result of the predicate!
+-- So, in order for this to qualify as a legal 'AffineTraversal' you can only use it for actions that preserve the result of the predicate!
 --
 -- >>> [1..10]^..folded.filtered even
 -- [2,4,6,8,10]
 --
 -- This will preserve an index if it is present.
-filtered :: (Choice p, Applicative f) => (a -> Bool) -> Overloaded' p f a a
+filtered :: (Choice p, Applicative f) => (a -> Bool) -> Optic' p f a a
 filtered p = dimap (\x -> if p x then Right x else Left x) (either pure id) . right'
 {-# INLINE filtered #-}
 
@@ -295,14 +325,10 @@ filtered p = dimap (\x -> if p x then Right x else Left x) (either pure id) . ri
 -- 'takingWhile' :: (a -> 'Bool') -> 'Lens'' s a                        -> 'Fold' s a -- * See note below
 -- 'takingWhile' :: (a -> 'Bool') -> 'Prism'' s a                       -> 'Fold' s a -- * See note below
 -- 'takingWhile' :: (a -> 'Bool') -> 'Iso'' s a                         -> 'Fold' s a -- * See note below
--- 'takingWhile' :: (a -> 'Bool') -> 'Action' m s a                     -> 'MonadicFold' m s a
--- 'takingWhile' :: (a -> 'Bool') -> 'MonadicFold' m s a                -> 'MonadicFold' m s a
 -- 'takingWhile' :: (a -> 'Bool') -> 'IndexedTraversal'' i s a          -> 'IndexedFold' i s a -- * See note below
 -- 'takingWhile' :: (a -> 'Bool') -> 'IndexedLens'' i s a               -> 'IndexedFold' i s a -- * See note below
 -- 'takingWhile' :: (a -> 'Bool') -> 'IndexedFold' i s a                -> 'IndexedFold' i s a
 -- 'takingWhile' :: (a -> 'Bool') -> 'IndexedGetter' i s a              -> 'IndexedFold' i s a
--- 'takingWhile' :: (a -> 'Bool') -> 'IndexedAction' i m s a            -> 'IndexedMonadicFold' i m s a
--- 'takingWhile' :: (a -> 'Bool') -> 'IndexedMonadicFold' i m s a       -> 'IndexedMonadicFold' i m s a
 -- @
 --
 -- /Note:/ When applied to a 'Traversal', 'takingWhile' yields something that can be used as if it were a 'Traversal', but
@@ -340,11 +366,6 @@ takingWhile p l pafb = fmap runMagma . traverse (corep pafb) . runTakingWhile .
 -- 'droppingWhile' :: (a -> 'Bool') -> 'IndexPreservingLens'' s a         -> 'IndexPreservingFold' s a -- see notes
 -- 'droppingWhile' :: (a -> 'Bool') -> 'IndexPreservingGetter' s a        -> 'IndexPreservingFold' s a
 -- 'droppingWhile' :: (a -> 'Bool') -> 'IndexPreservingFold' s a          -> 'IndexPreservingFold' s a
--- 'droppingWhile' :: (a -> 'Bool') -> 'IndexPreservingAction' m s a      -> 'IndexPreservingFold' s a
--- @
---
--- @
--- 'droppingWhile' :: (a -> 'Bool') -> 'IndexPreservingMonadicFold' m s a -> 'IndexPreservingMonadicFold' m s a
 -- @
 --
 -- @
@@ -352,11 +373,6 @@ takingWhile p l pafb = fmap runMagma . traverse (corep pafb) . runTakingWhile .
 -- 'droppingWhile' :: (a -> 'Bool') -> 'IndexedLens'' i s a               -> 'IndexedFold' i s a       -- see notes
 -- 'droppingWhile' :: (a -> 'Bool') -> 'IndexedGetter' i s a              -> 'IndexedFold' i s a
 -- 'droppingWhile' :: (a -> 'Bool') -> 'IndexedFold' i s a                -> 'IndexedFold' i s a
--- 'droppingWhile' :: (a -> 'Bool') -> 'IndexedAction' i m s a            -> 'IndexedFold' i s a
--- @
---
--- @
--- 'droppingWhile' :: (a -> 'Bool') -> 'IndexedMonadicFold' i m s a       -> 'IndexedMonadicFold' i m s a
 -- @
 --
 -- Note: Many uses of this combinator will yield something that meets the types, but not the laws of a valid
@@ -365,8 +381,8 @@ takingWhile p l pafb = fmap runMagma . traverse (corep pafb) . runTakingWhile .
 -- and 'Traversal' fusion is not sound.
 droppingWhile :: (Conjoined p, Profunctor q, Applicative f)
               => (a -> Bool)
-              -> Overloading p q (Compose (State Bool) f) s t a a
-              -> Overloading p q f s t a a
+              -> Optical p q (Compose (State Bool) f) s t a a
+              -> Optical p q f s t a a
 droppingWhile p l f = (flip evalState True .# getCompose) `rmap` l g where
   g = cotabulate $ \wa -> Compose $ state $ \b -> let
       a = extract wa
@@ -374,6 +390,44 @@ droppingWhile p l f = (flip evalState True .# getCompose) `rmap` l g where
     in (if b' then pure a else corep f wa, b')
 {-# INLINE droppingWhile #-}
 
+-- | A 'Fold' over the individual 'words' of a 'String'.
+--
+-- @
+-- 'worded' :: 'Fold' 'String' 'String'
+-- 'worded' :: 'Traversal'' 'String' 'String'
+-- @
+--
+-- @
+-- 'worded' :: 'IndexedFold' 'Int' 'String' 'String'
+-- 'worded' :: 'IndexedTraversal'' 'Int' 'String' 'String'
+-- @
+--
+-- Note: This function type-checks as a 'Traversal' but it doesn't satisfy the laws. It's only valid to use it
+-- when you don't insert any whitespace characters while traversing, and if your original 'String' contains only
+-- isolated space characters (and no other characters that count as space, such as non-breaking spaces).
+worded :: Applicative f => IndexedLensLike' Int f String String
+worded f = fmap unwords . conjoined traverse (indexing traverse) f . words
+{-# INLINE worded #-}
+
+-- | A 'Fold' over the individual 'lines' of a 'String'.
+--
+-- @
+-- 'lined' :: 'Fold' 'String' 'String'
+-- 'lined' :: 'Traversal'' 'String' 'String'
+-- @
+--
+-- @
+-- 'lined' :: 'IndexedFold' 'Int' 'String' 'String'
+-- 'lined' :: 'IndexedTraversal'' 'Int' 'String' 'String'
+-- @
+--
+-- Note: This function type-checks as a 'Traversal' but it doesn't satisfy the laws. It's only valid to use it
+-- when you don't insert any newline characters while traversing, and if your original 'String' contains only
+-- isolated newline characters.
+lined :: Applicative f => IndexedLensLike' Int f String String
+lined f = fmap (intercalate "\n") . conjoined traverse (indexing traverse) f . lines
+{-# INLINE lined #-}
+
 --------------------------
 -- Fold/Getter combinators
 --------------------------
@@ -400,7 +454,7 @@ droppingWhile p l f = (flip evalState True .# getCompose) `rmap` l g where
 -- 'foldMapOf' :: 'Getting' r s a -> (a -> r) -> s -> r
 -- @
 foldMapOf :: Profunctor p => Accessing p r s a -> p a r -> s -> r
-foldMapOf l f = runAccessor #. l (Accessor #. f)
+foldMapOf l f = getConst #. l (Const #. f)
 {-# INLINE foldMapOf #-}
 
 -- | @
@@ -420,7 +474,7 @@ foldMapOf l f = runAccessor #. l (Accessor #. f)
 -- 'foldOf' :: 'Monoid' m => 'Prism'' s m     -> s -> m
 -- @
 foldOf :: Getting a s a -> s -> a
-foldOf l = runAccessor #. l Accessor
+foldOf l = getConst #. l Const
 {-# INLINE foldOf #-}
 
 -- | Right-associative fold of parts of a structure that are viewed through a 'Lens', 'Getter', 'Fold' or 'Traversal'.
@@ -565,7 +619,7 @@ orOf l = getAny #. foldMapOf l Any
 -- >>> anyOf both (=='x') ('x','y')
 -- True
 -- >>> import Data.Data.Lens
--- >>> anyOf biplate (== "world") (((),2::Int),"hello",("world",11))
+-- >>> anyOf biplate (== "world") (((),2::Int),"hello",("world",11::Int))
 -- True
 --
 -- @
@@ -615,6 +669,29 @@ allOf :: Profunctor p => Accessing p All s a -> p a Bool -> s -> Bool
 allOf l f = getAll #. foldMapOf l (All #. f)
 {-# INLINE allOf #-}
 
+-- | Returns 'True' only if no targets of a 'Fold' satisfy a predicate.
+--
+-- >>> noneOf each (is _Nothing) (Just 3, Just 4, Just 5)
+-- True
+-- >>> noneOf (folded.folded) (<10) [[13,99,20],[3,71,42]]
+-- False
+--
+-- @
+-- 'inoneOf' l = 'noneOf' l '.' 'Indexed'
+-- @
+--
+-- @
+-- 'noneOf' :: 'Getter' s a     -> (a -> 'Bool') -> s -> 'Bool'
+-- 'noneOf' :: 'Fold' s a       -> (a -> 'Bool') -> s -> 'Bool'
+-- 'noneOf' :: 'Lens'' s a      -> (a -> 'Bool') -> s -> 'Bool'
+-- 'noneOf' :: 'Iso'' s a       -> (a -> 'Bool') -> s -> 'Bool'
+-- 'noneOf' :: 'Traversal'' s a -> (a -> 'Bool') -> s -> 'Bool'
+-- 'noneOf' :: 'Prism'' s a     -> (a -> 'Bool') -> s -> 'Bool'
+-- @
+noneOf :: Profunctor p => Accessing p Any s a -> p a Bool -> s -> Bool
+noneOf l f = not . anyOf l f
+{-# INLINE noneOf #-}
+
 -- | Calculate the 'Product' of every number targeted by a 'Fold'.
 --
 -- >>> productOf both (4,5)
@@ -946,7 +1023,7 @@ notElemOf l = allOf l . (/=)
 -- 'concatMapOf' :: 'Traversal'' s a -> (a -> [r]) -> s -> [r]
 -- @
 concatMapOf :: Profunctor p => Accessing p [r] s a -> p a [r] -> s -> [r]
-concatMapOf l ces = runAccessor #. l (Accessor #. ces)
+concatMapOf l ces = getConst #. l (Const #. ces)
 {-# INLINE concatMapOf #-}
 
 -- | Concatenate all of the lists targeted by a 'Fold' into a longer list.
@@ -967,7 +1044,7 @@ concatMapOf l ces = runAccessor #. l (Accessor #. ces)
 -- 'concatOf' :: 'Traversal'' s [r] -> s -> [r]
 -- @
 concatOf :: Getting [r] s [r] -> s -> [r]
-concatOf l = runAccessor #. l Accessor
+concatOf l = getConst #. l Const
 {-# INLINE concatOf #-}
 
 
@@ -1339,6 +1416,46 @@ findOf :: Conjoined p => Accessing p (Endo (Maybe a)) s a -> p a Bool -> s -> Ma
 findOf l p = foldrOf l (cotabulate $ \wa y -> if corep p wa then Just (extract wa) else y) Nothing
 {-# INLINE findOf #-}
 
+-- | The 'findMOf' function takes a 'Lens' (or 'Getter', 'Iso', 'Fold', or 'Traversal'),
+-- a monadic predicate and a structure and returns in the monad the leftmost element of the structure
+-- matching the predicate, or 'Nothing' if there is no such element.
+--
+-- >>>  findMOf each ( \x -> print ("Checking " ++ show x) >> return (even x)) (1,3,4,6)
+-- "Checking 1"
+-- "Checking 3"
+-- "Checking 4"
+-- Just 4
+--
+-- >>>  findMOf each ( \x -> print ("Checking " ++ show x) >> return (even x)) (1,3,5,7)
+-- "Checking 1"
+-- "Checking 3"
+-- "Checking 5"
+-- "Checking 7"
+-- Nothing
+--
+-- @
+-- 'findMOf' :: ('Monad' m, 'Getter' s a)     -> (a -> m 'Bool') -> s -> m ('Maybe' a)
+-- 'findMOf' :: ('Monad' m, 'Fold' s a)       -> (a -> m 'Bool') -> s -> m ('Maybe' a)
+-- 'findMOf' :: ('Monad' m, 'Iso'' s a)       -> (a -> m 'Bool') -> s -> m ('Maybe' a)
+-- 'findMOf' :: ('Monad' m, 'Lens'' s a)      -> (a -> m 'Bool') -> s -> m ('Maybe' a)
+-- 'findMOf' :: ('Monad' m, 'Traversal'' s a) -> (a -> m 'Bool') -> s -> m ('Maybe' a)
+-- @
+--
+-- @
+-- 'findMOf' 'folded' :: (Monad m, Foldable f) => (a -> m Bool) -> f a -> m (Maybe a)
+-- 'ifindMOf' l ≡ 'findMOf' l '.' 'Indexed'
+-- @
+--
+-- A simpler version that didn't permit indexing, would be:
+--
+-- @
+-- 'findMOf' :: Monad m => 'Getting' ('Endo' (m ('Maybe' a))) s a -> (a -> m 'Bool') -> s -> m ('Maybe' a)
+-- 'findMOf' l p = 'foldrOf' l (\a y -> p a >>= \x -> if x then return ('Just' a) else y) $ return 'Nothing'
+-- @
+findMOf :: (Monad m, Conjoined p) => Accessing p (Endo (m (Maybe a))) s a -> p a (m Bool) -> s -> m (Maybe a)
+findMOf l p = foldrOf l (cotabulate $ \wa y -> corep p wa >>= \r -> if r then return (Just (extract wa)) else y) $ return Nothing
+{-# INLINE findMOf #-}
+
 -- | A variant of 'foldrOf' that has no base case and thus may only be applied
 -- to lenses and structures such that the 'Lens' views at least one element of
 -- the structure.
@@ -1567,7 +1684,7 @@ hasn't l = getAll #. foldMapOf l (\_ -> All False)
 -- 'pre' :: 'Simple' 'Prism' s a     -> 'IndexPreservingGetter' s ('Maybe' a)
 -- @
 pre :: Getting (First a) s a -> IndexPreservingGetter s (Maybe a)
-pre l = dimap (getFirst . runAccessor #. l (Accessor #. First #. Just)) coerce
+pre l = dimap (getFirst . getConst #. l (Const #. First #. Just)) coerce
 {-# INLINE pre #-}
 
 -- | This converts an 'IndexedFold' to an 'IndexPreservingGetter' that returns the first index
@@ -1580,7 +1697,7 @@ pre l = dimap (getFirst . runAccessor #. l (Accessor #. First #. Just)) coerce
 -- 'ipre' :: 'Simple' ('IndexedLens' i) s a      -> 'IndexPreservingGetter' s ('Maybe' (i, a))
 -- @
 ipre :: IndexedGetting i (First (i, a)) s a -> IndexPreservingGetter s (Maybe (i, a))
-ipre l = dimap (getFirst . runAccessor #. l (Indexed $ \i a -> Accessor (First (Just (i, a))))) coerce
+ipre l = dimap (getFirst . getConst #. l (Indexed $ \i a -> Const (First (Just (i, a))))) coerce
 {-# INLINE ipre #-}
 
 ------------------------------------------------------------------------------
@@ -1738,7 +1855,7 @@ preuse :: MonadState s m => Getting (First a) s a -> m (Maybe a)
 preuse l = gets (preview l)
 {-# INLINE preuse #-}
 
--- | Retrieve the first index and value targeted by an 'IndexedFold' or 'IndexedTraversal' (or 'Just' the index 
+-- | Retrieve the first index and value targeted by an 'IndexedFold' or 'IndexedTraversal' (or 'Just' the index
 -- and result from an 'IndexedGetter' or 'IndexedLens') into the current state.
 --
 -- @
@@ -1804,7 +1921,7 @@ ipreuses l f = gets (ipreviews l f)
 --
 -- /NB:/ To write back through an 'Iso', you want to use 'Control.Lens.Isomorphic.from'.
 -- Similarly, to write back through an 'Prism', you want to use 'Control.Lens.Review.re'.
-backwards :: (Profunctor p, Profunctor q) => Overloading p q (Backwards f) s t a b -> Overloading p q f s t a b
+backwards :: (Profunctor p, Profunctor q) => Optical p q (Backwards f) s t a b -> Optical p q f s t a b
 backwards l f = forwards #. l (Backwards #. f)
 {-# INLINE backwards #-}
 
@@ -1908,6 +2025,25 @@ iallOf :: IndexedGetting i All s a -> (i -> a -> Bool) -> s -> Bool
 iallOf l = allOf l .# Indexed
 {-# INLINE iallOf #-}
 
+-- | Return whether or not none of the elements viewed through an 'IndexedFold' or 'IndexedTraversal'
+-- satisfy a predicate, with access to the @i@.
+--
+-- When you don't need access to the index then 'noneOf' is more flexible in what it accepts.
+--
+-- @
+-- 'noneOf' l ≡ 'inoneOf' l '.' 'const'
+-- @
+--
+-- @
+-- 'inoneOf' :: 'IndexedGetter' i s a     -> (i -> a -> 'Bool') -> s -> 'Bool'
+-- 'inoneOf' :: 'IndexedFold' i s a       -> (i -> a -> 'Bool') -> s -> 'Bool'
+-- 'inoneOf' :: 'IndexedLens'' i s a      -> (i -> a -> 'Bool') -> s -> 'Bool'
+-- 'inoneOf' :: 'IndexedTraversal'' i s a -> (i -> a -> 'Bool') -> s -> 'Bool'
+-- @
+inoneOf :: IndexedGetting i Any s a -> (i -> a -> Bool) -> s -> Bool
+inoneOf l = noneOf l .# Indexed
+{-# INLINE inoneOf #-}
+
 -- | Traverse the targets of an 'IndexedFold' or 'IndexedTraversal' with access to the @i@, discarding the results.
 --
 -- When you don't need access to the index then 'traverseOf_' is more flexible in what it accepts.
@@ -2011,7 +2147,7 @@ iconcatMapOf :: IndexedGetting i [r] s a -> (i -> a -> [r]) -> s -> [r]
 iconcatMapOf = ifoldMapOf
 {-# INLINE iconcatMapOf #-}
 
--- | The 'findOf' function takes an 'IndexedFold' or 'IndexedTraversal', a predicate that is also
+-- | The 'ifindOf' function takes an 'IndexedFold' or 'IndexedTraversal', a predicate that is also
 -- supplied the index, a structure and returns the left-most element of the structure
 -- matching the predicate, or 'Nothing' if there is no such element.
 --
@@ -2031,6 +2167,26 @@ ifindOf :: IndexedGetting i (Endo (Maybe a)) s a -> (i -> a -> Bool) -> s -> May
 ifindOf l = findOf l .# Indexed
 {-# INLINE ifindOf #-}
 
+-- | The 'ifindMOf' function takes an 'IndexedFold' or 'IndexedTraversal', a monadic predicate that is also
+-- supplied the index, a structure and returns in the monad the left-most element of the structure
+-- matching the predicate, or 'Nothing' if there is no such element.
+--
+-- When you don't need access to the index then 'findMOf' is more flexible in what it accepts.
+--
+-- @
+-- 'findMOf' l ≡ 'ifindMOf' l '.' 'const'
+-- @
+--
+-- @
+-- 'ifindMOf' :: 'Monad' m => 'IndexedGetter' i s a     -> (i -> a -> m 'Bool') -> s -> m ('Maybe' a)
+-- 'ifindMOf' :: 'Monad' m => 'IndexedFold' i s a       -> (i -> a -> m 'Bool') -> s -> m ('Maybe' a)
+-- 'ifindMOf' :: 'Monad' m => 'IndexedLens'' i s a      -> (i -> a -> m 'Bool') -> s -> m ('Maybe' a)
+-- 'ifindMOf' :: 'Monad' m => 'IndexedTraversal'' i s a -> (i -> a -> m 'Bool') -> s -> m ('Maybe' a)
+-- @
+ifindMOf :: Monad m => IndexedGetting i (Endo (m (Maybe a))) s a -> (i -> a -> m Bool) -> s -> m (Maybe a)
+ifindMOf l = findMOf l .# Indexed
+{-# INLINE ifindMOf #-}
+
 -- | /Strictly/ fold right over the elements of a structure with an index.
 --
 -- When you don't need access to the index then 'foldrOf'' is more flexible in what it accepts.
@@ -2178,7 +2334,7 @@ s ^@?! l = ifoldrOf l (\i x _ -> (i,x)) (error "(^@?!): empty Fold") s
 -- access to both the value and the index.
 --
 -- Note: As with 'filtered', this is /not/ a legal 'IndexedTraversal', unless you are very careful not to invalidate the predicate on the target!
-ifiltered :: (Indexable i p, Applicative f) => (i -> a -> Bool) -> Overloading' p (Indexed i) f a a
+ifiltered :: (Indexable i p, Applicative f) => (i -> a -> Bool) -> Optical' p (Indexed i) f a a
 ifiltered p f = Indexed $ \i a -> if p i a then indexed f i a else pure a
 {-# INLINE ifiltered #-}
 
@@ -2190,15 +2346,13 @@ ifiltered p f = Indexed $ \i a -> if p i a then indexed f i a else pure a
 -- 'itakingWhile' :: (i -> a -> 'Bool') -> 'IndexedTraversal'' i s a    -> 'IndexedFold' i s a
 -- 'itakingWhile' :: (i -> a -> 'Bool') -> 'IndexedLens'' i s a         -> 'IndexedFold' i s a
 -- 'itakingWhile' :: (i -> a -> 'Bool') -> 'IndexedGetter' i s a        -> 'IndexedFold' i s a
--- 'itakingWhile' :: (i -> a -> 'Bool') -> 'IndexedMonadicFold' i m s a -> 'IndexedMonadicFold' i m s a
--- 'itakingWhile' :: (i -> a -> 'Bool') -> 'IndexedAction' i m s a      -> 'IndexedMonadicFold' i m s a
 -- @
 itakingWhile :: (Indexable i p, Profunctor q, Contravariant f, Applicative f)
          => (i -> a -> Bool)
-         -> Overloading (Indexed i) q (Accessor (Endo (f s))) s s a a
-         -> Overloading p q f s s a a
-itakingWhile p l f = (flip appEndo noEffect .# runAccessor) `rmap` l g where
-  g = Indexed $ \i a -> Accessor . Endo $ if p i a then (indexed f i a *>) else const noEffect
+         -> Optical (Indexed i) q (Const (Endo (f s))) s s a a
+         -> Optical p q f s s a a
+itakingWhile p l f = (flip appEndo noEffect .# getConst) `rmap` l g where
+  g = Indexed $ \i a -> Const . Endo $ if p i a then (indexed f i a *>) else const noEffect
 {-# INLINE itakingWhile #-}
 
 -- | Obtain an 'IndexedFold' by dropping elements from another 'IndexedFold', 'IndexedLens', 'IndexedGetter' or 'IndexedTraversal' while a predicate holds.
@@ -2208,8 +2362,6 @@ itakingWhile p l f = (flip appEndo noEffect .# runAccessor) `rmap` l g where
 -- 'idroppingWhile' :: (i -> a -> 'Bool') -> 'IndexedTraversal'' i s a    -> 'IndexedFold' i s a -- see notes
 -- 'idroppingWhile' :: (i -> a -> 'Bool') -> 'IndexedLens'' i s a         -> 'IndexedFold' i s a -- see notes
 -- 'idroppingWhile' :: (i -> a -> 'Bool') -> 'IndexedGetter' i s a        -> 'IndexedFold' i s a
--- 'idroppingWhile' :: (i -> a -> 'Bool') -> 'IndexedMonadicFold' i m s a -> 'IndexedMonadicFold' i m s a
--- 'idroppingWhile' :: (i -> a -> 'Bool') -> 'IndexedAction' i m s a      -> 'IndexedMonadicFold' i m s a
 -- @
 --
 -- Applying 'idroppingWhile' to an 'IndexedLens' or 'IndexedTraversal' will still allow you to use it as a
@@ -2217,8 +2369,8 @@ itakingWhile p l f = (flip appEndo noEffect .# runAccessor) `rmap` l g where
 -- 'True', then you will break the 'Traversal' laws and 'Traversal' fusion will no longer be sound.
 idroppingWhile :: (Indexable i p, Profunctor q, Applicative f)
               => (i -> a -> Bool)
-              -> Overloading (Indexed i) q (Compose (State Bool) f) s t a a
-              -> Overloading p q f s t a a
+              -> Optical (Indexed i) q (Compose (State Bool) f) s t a a
+              -> Optical p q f s t a a
 idroppingWhile p l f = (flip evalState True .# getCompose) `rmap` l g where
   g = Indexed $ \ i a -> Compose $ state $ \b -> let
       b' = b && p i a
@@ -2242,3 +2394,19 @@ headOf l = getFirst #. foldMapOf l (First #. Just)
 skip :: a -> ()
 skip _ = ()
 {-# INLINE skip #-}
+
+------------------------------------------------------------------------------
+-- Folds with Reified Monoid
+------------------------------------------------------------------------------
+
+foldBy :: Foldable t => (a -> a -> a) -> a -> t a -> a
+foldBy f z = reifyFold f z (foldMap M)
+
+foldByOf :: (forall i. Getting (M a i) s a) -> (a -> a -> a) -> a -> s -> a
+foldByOf l f z = reifyFold f z (foldMapOf l M)
+
+foldMapBy :: Foldable t => (r -> r -> r) -> r -> (a -> r) -> t a -> r
+foldMapBy f z g = reifyFold f z (foldMap (M #. g))
+
+foldMapByOf :: (forall s. Getting (M r s) t a) -> (r -> r -> r) -> r -> (a -> r) -> t -> r
+foldMapByOf l f z g = reifyFold f z (foldMapOf l (M #. g))
