@@ -12,7 +12,7 @@
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Getter
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -21,17 +21,16 @@
 --
 -- A @'Getter' s a@ is just any function @(s -> a)@, which we've flipped
 -- into continuation passing style, @(a -> r) -> s -> r@ and decorated
--- with 'Accessor' to obtain:
+-- with 'Const' to obtain:
 --
--- @type 'Getting' r s t a b = (a -> 'Accessor' r b) -> s -> 'Accessor' r t@
+-- @type 'Getting' r s a = (a -> 'Const' r a) -> s -> 'Const' r s@
 --
--- If we restrict access to knowledge about the type 'r' and can work for
--- any b and t, we could get:
+-- If we restrict access to knowledge about the type 'r', we could get:
 --
--- @type 'Getter' s a = forall r. 'Getting' r s s a a@
+-- @type 'Getter' s a = forall r. 'Getting' r s a@
 --
--- But we actually hide the use of 'Accessor' behind a class 'Gettable'
--- to error messages from type class resolution rather than at unification
+-- But we actually hide the use of 'Const' behind a class 'Gettable' to
+-- report error messages from type class resolution rather than at unification
 -- time, where they are much uglier.
 --
 -- @type 'Getter' s a = forall f. 'Gettable' f => (a -> f a) -> s -> f s@
@@ -52,6 +51,7 @@ module Control.Lens.Getter
   , Accessing
   -- * Building Getters
   , to
+  , ito
   -- * Combinators for Getters and Folds
   , (^.)
   , view, views
@@ -66,10 +66,11 @@ module Control.Lens.Getter
   -- * Implementation Details
   , Contravariant(..)
   , coerce, coerced
-  , Accessor(..)
+  , Const(..)
   , Gettable
   ) where
 
+import Control.Applicative
 import Control.Lens.Internal.Getter
 import Control.Lens.Internal.Indexed
 import Control.Lens.Type
@@ -117,9 +118,13 @@ infixl 8 ^., ^@.
 -- >>> (0, -5)^._2.to abs
 -- 5
 to :: (s -> a) -> IndexPreservingGetter s a
-to f = dimap f coerce
+to k = dimap k coerce
 {-# INLINE to #-}
 
+ito :: (s -> (i, a)) -> IndexedGetter i s a
+ito k = dimap k coerce . uncurry . indexed
+{-# INLINE ito #-}
+
 -- | When you see this in a type signature it indicates that you can
 -- pass the function a 'Lens', 'Getter',
 -- 'Control.Lens.Traversal.Traversal', 'Control.Lens.Fold.Fold',
@@ -133,18 +138,18 @@ to f = dimap f coerce
 -- 'Control.Lens.Iso.Iso' we also restricted choices of the irrelevant @t@ and
 -- @b@ parameters.
 --
--- If a function accepts a @'Getting' r s t a b@, then when @r@ is a 'Data.Monoid.Monoid', then
+-- If a function accepts a @'Getting' r s a@, then when @r@ is a 'Data.Monoid.Monoid', then
 -- you can pass a 'Control.Lens.Fold.Fold' (or
 -- 'Control.Lens.Traversal.Traversal'), otherwise you can only pass this a
 -- 'Getter' or 'Lens'.
-type Getting r s a = (a -> Accessor r a) -> s -> Accessor r s
+type Getting r s a = (a -> Const r a) -> s -> Const r s
 
 -- | Used to consume an 'Control.Lens.Fold.IndexedFold'.
-type IndexedGetting i m s a = Indexed i a (Accessor m a) -> s -> Accessor m s
+type IndexedGetting i m s a = Indexed i a (Const m a) -> s -> Const m s
 
 -- | This is a convenient alias used when consuming (indexed) getters and (indexed) folds
 -- in a highly general fashion.
-type Accessing p m s a = p a (Accessor m a) -> s -> Accessor m s
+type Accessing p m s a = p a (Const m a) -> s -> Const m s
 
 -------------------------------------------------------------------------------
 -- Getting Values
@@ -193,7 +198,7 @@ type Accessing p m s a = p a (Accessor m a) -> s -> Accessor m s
 -- 'view' :: ('MonadReader' s m, 'Data.Monoid.Monoid' a) => 'Control.Lens.Traversal.Traversal'' s a -> m a
 -- @
 view :: MonadReader s m => Getting a s a -> m a
-view l = Reader.asks (runAccessor #. l Accessor)
+view l = Reader.asks (getConst #. l Const)
 {-# INLINE view #-}
 
 -- | View a function of the value pointed to by a 'Getter' or 'Lens' or the result of
@@ -234,8 +239,8 @@ view l = Reader.asks (runAccessor #. l Accessor)
 -- @
 -- 'views' :: 'MonadReader' s m => 'Getting' r s a -> (a -> r) -> m r
 -- @
-views :: (Profunctor p, MonadReader s m) => Overloading p (->) (Accessor r) s s a a -> p a r -> m r
-views l f = Reader.asks (runAccessor #. l (Accessor #. f))
+views :: (Profunctor p, MonadReader s m) => Optical p (->) (Const r) s s a a -> p a r -> m r
+views l f = Reader.asks (getConst #. l (Const #. f))
 {-# INLINE views #-}
 
 -- | View the value pointed to by a 'Getter' or 'Lens' or the
@@ -265,7 +270,7 @@ views l f = Reader.asks (runAccessor #. l (Accessor #. f))
 -- ('^.') :: 'Data.Monoid.Monoid' m => s -> 'Control.Lens.Traversal.Traversal'' s m -> m
 -- @
 (^.) :: s -> Getting a s a -> a
-s ^. l = runAccessor (l Accessor s)
+s ^. l = getConst (l Const s)
 {-# INLINE (^.) #-}
 
 -------------------------------------------------------------------------------
@@ -313,7 +318,7 @@ use l = State.gets (view l)
 -- @
 -- 'uses' :: 'MonadState' s m => 'Getting' r s t a b -> (a -> r) -> m r
 -- @
-uses :: (Profunctor p, MonadState s m) => Overloading p (->) (Accessor r) s s a a -> p a r -> m r
+uses :: (Profunctor p, MonadState s m) => Optical p (->) (Const r) s s a a -> p a r -> m r
 uses l f = State.gets (views l f)
 {-# INLINE uses #-}
 
@@ -398,7 +403,7 @@ ilistenings l iuv m = do
 -- When applied to an 'IndexedFold' the result will most likely be a nonsensical monoidal summary of
 -- the indices tupled with a monoidal summary of the values and probably not whatever it is you wanted.
 iview :: MonadReader s m => IndexedGetting i (i,a) s a -> m (i,a)
-iview l = asks (runAccessor #. l (Indexed $ \i -> Accessor #. (,) i))
+iview l = asks (getConst #. l (Indexed $ \i -> Const #. (,) i))
 {-# INLINE iview #-}
 
 -- | View a function of the index and value of an 'IndexedGetter' into the current environment.
@@ -417,7 +422,7 @@ iviews l = views l .# Indexed
 -- When applied to an 'IndexedFold' the result will most likely be a nonsensical monoidal summary of
 -- the indices tupled with a monoidal summary of the values and probably not whatever it is you wanted.
 iuse :: MonadState s m => IndexedGetting i (i,a) s a -> m (i,a)
-iuse l = gets (runAccessor #. l (Indexed $ \i -> Accessor #. (,) i))
+iuse l = gets (getConst #. l (Indexed $ \i -> Const #. (,) i))
 {-# INLINE iuse #-}
 
 -- | Use a function of the index and value of an 'IndexedGetter' into the current state.
@@ -427,19 +432,13 @@ iuses :: MonadState s m => IndexedGetting i r s a -> (i -> a -> r) -> m r
 iuses l = uses l .# Indexed
 {-# INLINE iuses #-}
 
--- | View the value pointed to by a 'Getter' or 'Lens'.
+-- | View the index and value of an 'IndexedGetter' or 'IndexedLens'.
 --
 -- This is the same operation as 'iview' with the arguments flipped.
 --
 -- The fixity and semantics are such that subsequent field accesses can be
 -- performed with ('Prelude..').
 --
--- >>> (a,b,c,d)^@._2
--- (1,b)
---
--- >>> ("hello","world","!!!")^@._2
--- (1,"world")
---
 -- @
 -- ('^@.') :: s -> 'IndexedGetter' i s a -> (i, a)
 -- ('^@.') :: s -> 'IndexedLens'' i s a  -> (i, a)
@@ -447,7 +446,7 @@ iuses l = uses l .# Indexed
 --
 -- The result probably doesn't have much meaning when applied to an 'IndexedFold'.
 (^@.) :: s -> IndexedGetting i (i, a) s a -> (i, a)
-s ^@. l = runAccessor $ l (Indexed $ \i -> Accessor #. (,) i) s
+s ^@. l = getConst $ l (Indexed $ \i -> Const #. (,) i) s
 {-# INLINE (^@.) #-}
 
 -- | Coerce a 'Gettable' 'LensLike' to a 'Simple' 'LensLike'. This is useful
