@@ -3,15 +3,11 @@
 {-# LANGUAGE Rank2Types #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE DefaultSignatures #-}
 {-# LANGUAGE UndecidableInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE FunctionalDependencies #-}
-#ifdef DEFAULT_SIGNATURES
-{-# LANGUAGE DefaultSignatures #-}
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 706
-#define MPTC_DEFAULTS
-#endif
-#endif
+
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-} -- vector, hashable
 #endif
@@ -22,7 +18,7 @@
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Indexed
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -45,10 +41,10 @@ module Control.Lens.Indexed
   , FunctorWithIndex(..)
   -- * Indexed Foldables
   , FoldableWithIndex(..)
-  , ifolding
   -- ** Indexed Foldable Combinators
   , iany
   , iall
+  , inone, none
   , itraverse_
   , ifor_
   , imapM_
@@ -72,6 +68,9 @@ module Control.Lens.Indexed
   , iforM
   , imapAccumR
   , imapAccumL
+  -- * Indexed Folds with Reified Monoid
+  , ifoldMapBy
+  , ifoldMapByOf
   ) where
 
 import Control.Applicative
@@ -79,8 +78,8 @@ import Control.Applicative.Backwards
 import Control.Monad (void, liftM)
 import Control.Monad.Trans.State.Lazy as Lazy
 import Control.Lens.Fold
+import Control.Lens.Getter
 import Control.Lens.Internal.Fold
-import Control.Lens.Internal.Getter
 import Control.Lens.Internal.Indexed
 import Control.Lens.Internal.Level
 import Control.Lens.Internal.Magma
@@ -88,12 +87,11 @@ import Control.Lens.Setter
 import Control.Lens.Traversal
 import Control.Lens.Type
 import Data.Foldable
-import Data.Functor.Contravariant
-import Data.Functor.Identity
 import Data.Functor.Reverse
 import Data.Hashable
 import Data.HashMap.Lazy as HashMap
 import Data.IntMap as IntMap
+import Data.List.NonEmpty as NonEmpty
 import Data.Map as Map
 import Data.Monoid
 import Data.Profunctor.Unsafe
@@ -120,7 +118,7 @@ infixr 9 <.>, <., .>
 --
 -- Mnemonically, the @>@ points to the indexing we want to preserve.
 --
--- This is the same as @('.')@. 
+-- This is the same as @('.')@.
 --
 -- @f '.' g@ (and @f '.>' g@) gives you the index of @g@ unless @g@ is index-preserving, like a
 -- 'Prism', 'Iso' or 'Equality', in which case it'll pass through the index of @f@.
@@ -153,13 +151,13 @@ icompose ijk istr jabst cab = istr . Indexed $ \i -> jabst . Indexed $ \j -> ind
 --
 -- The result is only valid to compose in a 'Traversal', if you don't edit the
 -- index as edits to the index have no effect.
-withIndex :: (Indexable i p, Functor f) => Overloading p (Indexed i) f s t (i, s) (j, t)
+withIndex :: (Indexable i p, Functor f) => Optical p (Indexed i) f s t (i, s) (j, t)
 withIndex f = Indexed $ \i a -> snd <$> indexed f i (i, a)
 {-# INLINE withIndex #-}
 
 -- | When composed with an 'IndexedFold' or 'IndexedTraversal' this yields an
 -- ('Indexed') 'Fold' of the indices.
-asIndex :: (Indexable i p, Contravariant f, Functor f) => Overloading' p (Indexed i) f s i
+asIndex :: (Indexable i p, Contravariant f, Functor f) => Optical' p (Indexed i) f s i
 asIndex f = Indexed $ \i _ -> coerce (indexed f i i)
 {-# INLINE asIndex #-}
 
@@ -175,7 +173,7 @@ asIndex f = Indexed $ \i _ -> coerce (indexed f i i)
 --
 -- >>> over (traversed.indices (>0)) Prelude.reverse $ ["He","was","stressed","o_O"]
 -- ["He","saw","desserts","O_o"]
-indices :: (Indexable i p, Applicative f) => (i -> Bool) -> Overloading' p (Indexed i) f a a
+indices :: (Indexable i p, Applicative f) => (i -> Bool) -> Optical' p (Indexed i) f a a
 indices p f = Indexed $ \i a -> if p i then indexed f i a else pure a
 {-# INLINE indices #-}
 
@@ -183,7 +181,7 @@ indices p f = Indexed $ \i a -> if p i then indexed f i a else pure a
 --
 -- >>> ["hello","the","world","!!!"]^?traversed.index 2
 -- Just "world"
-index :: (Indexable i p, Eq i, Applicative f) => i -> Overloading' p (Indexed i) f a a
+index :: (Indexable i p, Eq i, Applicative f) => i -> Optical' p (Indexed i) f a a
 index j f = Indexed $ \i a -> if j == i then indexed f i a else pure a
 {-# INLINE index #-}
 
@@ -203,7 +201,7 @@ index j f = Indexed $ \i a -> if j == i then indexed f i a else pure a
 class Functor f => FunctorWithIndex i f | f -> i where
   -- | Map with access to the index.
   imap :: (i -> a -> b) -> f a -> f b
-#ifdef MPTC_DEFAULTS
+#ifndef HLINT
   default imap :: TraversableWithIndex i f => (i -> a -> b) -> f a -> f b
   imap = iover itraversed
   {-# INLINE imap #-}
@@ -212,7 +210,7 @@ class Functor f => FunctorWithIndex i f | f -> i where
   -- | The 'IndexedSetter' for a 'FunctorWithIndex'.
   --
   -- If you don't need access to the index, then 'mapped' is more flexible in what it accepts.
-  imapped :: FunctorWithIndex i f => IndexedSetter i (f a) (f b) a b
+  imapped :: IndexedSetter i (f a) (f b) a b
   imapped = conjoined mapped (isets imap)
   {-# INLINE imapped #-}
 
@@ -231,7 +229,7 @@ class Foldable f => FoldableWithIndex i f | f -> i where
   -- 'foldMap' ≡ 'ifoldMap' '.' 'const'
   -- @
   ifoldMap :: Monoid m => (i -> a -> m) -> f a -> m
-#ifdef MPTC_DEFAULTS
+#ifndef HLINT
   default ifoldMap :: (TraversableWithIndex i f, Monoid m) => (i -> a -> m) -> f a -> m
   ifoldMap = ifoldMapOf itraversed
   {-# INLINE ifoldMap #-}
@@ -288,13 +286,6 @@ class Foldable f => FoldableWithIndex i f | f -> i where
     where f' i x k z = k $! f i z x
   {-# INLINE ifoldl' #-}
 
--- | Obtain a 'Fold' by lifting an operation that returns a 'Foldable' result.
---
--- This can be useful to lift operations from @'Data.List'@ and elsewhere into a 'Fold'.
-ifolding :: FoldableWithIndex i f => (s -> f a) -> IndexedFold i s a
-ifolding sfa iagb = coerce . itraverse_ (indexed iagb) . sfa
-{-# INLINE ifolding #-}
-
 -- | Return whether or not any element in a container satisfies a predicate, with access to the index @i@.
 --
 -- When you don't need access to the index then 'any' is more flexible in what it accepts.
@@ -317,6 +308,27 @@ iall :: FoldableWithIndex i f => (i -> a -> Bool) -> f a -> Bool
 iall f = getAll #. ifoldMap (\i -> All #. f i)
 {-# INLINE iall #-}
 
+-- | Return whether or not none of the elements in a container satisfy a predicate, with access to the index @i@.
+--
+-- When you don't need access to the index then 'none' is more flexible in what it accepts.
+--
+-- @
+-- 'none' ≡ 'inone' '.' 'const'
+-- 'inone' f ≡ 'not' '.' 'iany' f
+-- @
+inone :: FoldableWithIndex i f => (i -> a -> Bool) -> f a -> Bool
+inone f = not . iany f
+{-# INLINE inone #-}
+
+-- | Determines whether no elements of the structure satisfy the predicate.
+--
+-- @
+-- 'none' f ≡ 'not' '.' 'any' f
+-- @
+none :: Foldable f => (a -> Bool) -> f a -> Bool
+none f = not . Data.Foldable.any f
+{-# INLINE none #-}
+
 -- | Traverse elements with access to the index @i@, discarding the results.
 --
 -- When you don't need access to the index then 'traverse_' is more flexible in what it accepts.
@@ -445,7 +457,7 @@ itoList = ifoldr (\i c -> ((i,c):)) []
 class (FunctorWithIndex i t, FoldableWithIndex i t, Traversable t) => TraversableWithIndex i t | t -> i where
   -- | Traverse an indexed container.
   itraverse :: Applicative f => (i -> a -> f b) -> t a -> f (t b)
-#ifdef MPTC_DEFAULTS
+#ifndef HLINT
   default itraverse :: Applicative f => (Int -> a -> f b) -> t a -> f (t b)
   itraverse = traversed .# Indexed
   {-# INLINE itraverse #-}
@@ -566,16 +578,18 @@ instance TraversableWithIndex k ((,) k) where
   {-# INLINE itraverse #-}
 
 -- | The position in the list is available as the index.
-instance FunctorWithIndex Int [] where
-  imap = iover itraversed
-  {-# INLINE imap #-}
-instance FoldableWithIndex Int [] where
-  ifoldMap = ifoldMapOf itraversed
-  {-# INLINE ifoldMap #-}
+instance FunctorWithIndex Int []
+instance FoldableWithIndex Int []
 instance TraversableWithIndex Int [] where
   itraverse = itraverseOf traversed
   {-# INLINE itraverse #-}
 
+instance FunctorWithIndex Int NonEmpty
+instance FoldableWithIndex Int NonEmpty
+instance TraversableWithIndex Int NonEmpty where
+  itraverse = itraverseOf traversed
+  {-# INLINE itraverse #-}
+
 instance FunctorWithIndex () Maybe where
   imap f = fmap (f ())
   {-# INLINE imap #-}
@@ -587,12 +601,8 @@ instance TraversableWithIndex () Maybe where
   {-# INLINE itraverse #-}
 
 -- | The position in the 'Seq' is available as the index.
-instance FunctorWithIndex Int Seq where
-  imap = iover itraversed
-  {-# INLINE imap #-}
-instance FoldableWithIndex Int Seq where
-  ifoldMap = ifoldMapOf itraversed
-  {-# INLINE ifoldMap #-}
+instance FunctorWithIndex Int Seq
+instance FoldableWithIndex Int Seq
 instance TraversableWithIndex Int Seq where
   itraverse = itraverseOf traversed
   {-# INLINE itraverse #-}
@@ -601,8 +611,6 @@ instance FunctorWithIndex Int Vector where
   imap = V.imap
   {-# INLINE imap #-}
 instance FoldableWithIndex Int Vector where
-  ifoldMap = ifoldMapOf itraversed
-  {-# INLINE ifoldMap #-}
   ifoldr = V.ifoldr
   {-# INLINE ifoldr #-}
   ifoldl = V.ifoldl . flip
@@ -615,12 +623,8 @@ instance TraversableWithIndex Int Vector where
   itraverse f = sequenceA . V.imap f
   {-# INLINE itraverse #-}
 
-instance FunctorWithIndex Int IntMap where
-  imap = iover itraversed
-  {-# INLINE imap #-}
-instance FoldableWithIndex Int IntMap where
-  ifoldMap = ifoldMapOf itraversed
-  {-# INLINE ifoldMap #-}
+instance FunctorWithIndex Int IntMap
+instance FoldableWithIndex Int IntMap
 instance TraversableWithIndex Int IntMap where
 #if MIN_VERSION_containers(0,5,0)
   itraverse = IntMap.traverseWithKey
@@ -629,12 +633,8 @@ instance TraversableWithIndex Int IntMap where
 #endif
   {-# INLINE itraverse #-}
 
-instance FunctorWithIndex k (Map k) where
-  imap = iover itraversed
-  {-# INLINE imap #-}
-instance FoldableWithIndex k (Map k) where
-  ifoldMap = ifoldMapOf itraversed
-  {-# INLINE ifoldMap #-}
+instance FunctorWithIndex k (Map k)
+instance FoldableWithIndex k (Map k)
 instance TraversableWithIndex k (Map k) where
 #if MIN_VERSION_containers(0,5,0)
   itraverse = Map.traverseWithKey
@@ -643,12 +643,8 @@ instance TraversableWithIndex k (Map k) where
 #endif
   {-# INLINE itraverse #-}
 
-instance (Eq k, Hashable k) => FunctorWithIndex k (HashMap k) where
-  imap = iover itraversed
-  {-# INLINE imap #-}
-instance (Eq k, Hashable k) => FoldableWithIndex k (HashMap k) where
-  ifoldMap = ifoldMapOf itraversed
-  {-# INLINE ifoldMap #-}
+instance (Eq k, Hashable k) => FunctorWithIndex k (HashMap k)
+instance (Eq k, Hashable k) => FoldableWithIndex k (HashMap k)
 instance (Eq k, Hashable k) => TraversableWithIndex k (HashMap k) where
   itraverse = HashMap.traverseWithKey
   {-# INLINE itraverse #-}
@@ -707,3 +703,12 @@ skip :: a -> ()
 skip _ = ()
 {-# INLINE skip #-}
 
+-------------------------------------------------------------------------------
+-- Indexed Folds with Reified Monoid
+-------------------------------------------------------------------------------
+
+ifoldMapBy :: FoldableWithIndex i t => (r -> r -> r) -> r -> (i -> a -> r) -> t a -> r
+ifoldMapBy f z g = reifyFold f z (ifoldMap (\i a -> M (g i a)))
+
+ifoldMapByOf :: (forall s. IndexedGetting i (M r s) t a) -> (r -> r -> r) -> r -> (i -> a -> r) -> t -> r
+ifoldMapByOf l f z g = reifyFold f z (ifoldMapOf l (\i a -> M (g i a)))
