@@ -11,7 +11,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.Action
--- Copyright   :  (C) 2012-2013 Edward Kmett
+-- Copyright   :  (C) 2012-2014 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -27,7 +27,6 @@ module Control.Lens.Internal.Action
 
 import Control.Applicative
 import Control.Applicative.Backwards
-import Control.Lens.Internal.Getter
 import Control.Monad
 import Data.Functor.Bind
 import Data.Functor.Contravariant
@@ -52,10 +51,10 @@ instance Effective m r f => Effective m (Dual r) (Backwards f) where
   ineffective = liftM Dual . ineffective . forwards
   {-# INLINE ineffective #-}
 
-instance Effective Identity r (Accessor r) where
-  effective = Accessor #. runIdentity
+instance Effective Identity r (Const r) where
+  effective = Const #. runIdentity
   {-# INLINE effective #-}
-  ineffective = Identity #. runAccessor
+  ineffective = Identity #. getConst
   {-# INLINE ineffective #-}
 
 ------------------------------------------------------------------------------
@@ -64,6 +63,7 @@ instance Effective Identity r (Accessor r) where
 
 -- | Wrap a monadic effect with a phantom type argument.
 newtype Effect m r a = Effect { getEffect :: m r }
+-- type role Effect representational nominal phantom
 
 instance Functor (Effect m r) where
   fmap _ (Effect m) = Effect m
