@@ -6,13 +6,16 @@
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE FunctionalDependencies #-}
+#if __GLASGOW_HASKELL__ >= 707
+{-# LANGUAGE RoleAnnotations #-}
+#endif
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.Context
--- Copyright   :  (C) 2012-2013 Edward Kmett
+-- Copyright   :  (C) 2012-2014 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -84,7 +87,7 @@ class IndexedFunctor w => IndexedComonad w where
 -- IndexedComonadStore
 ------------------------------------------------------------------------------
 
--- | This is an indexed analogue to 'ComonadStore' for when you are working with an 
+-- | This is an indexed analogue to 'ComonadStore' for when you are working with an
 -- 'IndexedComonad'.
 class IndexedComonad w => IndexedComonadStore w where
   -- | This is the generalization of 'pos' to an indexed comonad store.
@@ -139,6 +142,7 @@ class Corepresentable p => Sellable p w | w -> p where
 --
 -- A 'Context' is like a 'Control.Lens.Lens.Lens' that has already been applied to a some structure.
 data Context a b t = Context (b -> t) a
+-- type role Context representational representational representational
 
 instance IndexedFunctor Context where
   ifmap f (Context g t) = Context (f . g) t
@@ -209,6 +213,7 @@ type Context' a = Context a a
 -- impact on its performance, and which permits the use of an arbitrary 'Conjoined'
 -- 'Profunctor'
 newtype Pretext p a b t = Pretext { runPretext :: forall f. Functor f => p a (f b) -> f t }
+-- type role Pretext representational nominal nominal nominal
 
 -- | @type 'Pretext'' p a s = 'Pretext' p a a s@
 type Pretext' p a = Pretext p a a
@@ -271,6 +276,8 @@ instance Corepresentable p => Sellable p (Pretext p) where
 -- PretextT
 ------------------------------------------------------------------------------
 
+
+
 -- | This is a generalized form of 'Context' that can be repeatedly cloned with less
 -- impact on its performance, and which permits the use of an arbitrary 'Conjoined'
 -- 'Profunctor'.
@@ -281,6 +288,12 @@ instance Corepresentable p => Sellable p (Pretext p) where
 -- or 'Control.Lens.Action.Action'.
 newtype PretextT p (g :: * -> *) a b t = PretextT { runPretextT :: forall f. Functor f => p a (f b) -> f t }
 
+#if __GLASGOW_HASKELL__ >= 707
+-- really we want PretextT p g a b t to permit the last 3 arguments to be representational iff p and f accept representational arguments
+-- but that isn't currently an option in GHC
+type role PretextT representational nominal nominal nominal nominal
+#endif
+
 -- | @type 'PretextT'' p g a s = 'PretextT' p g a a s@
 type PretextT' p g a = PretextT p g a a
 
