@@ -9,7 +9,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.Deque
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -26,11 +26,11 @@ module Control.Lens.Internal.Deque
   ) where
 
 import Control.Applicative
-import Control.Lens.Combinators
 import Control.Lens.Cons
 import Control.Lens.Fold
 import Control.Lens.Indexed hiding ((<.>))
 import Control.Lens.Iso
+import Control.Lens.Lens
 import Control.Lens.Prism
 import Control.Monad
 import Data.Foldable as Foldable
@@ -186,7 +186,7 @@ check lf f lr r
   | otherwise = BD lf f lr r
 {-# INLINE check #-}
 
-instance (Choice p, Applicative f) => Cons p f (Deque a) (Deque b) a b where
+instance Cons (Deque a) (Deque b) a b where
   _Cons = prism (\(x,BD lf f lr r) -> check (lf + 1) (x : f) lr r) $ \ (BD lf f lr r) ->
     if lf + lr == 0
     then Left empty
@@ -195,7 +195,7 @@ instance (Choice p, Applicative f) => Cons p f (Deque a) (Deque b) a b where
       (x:xs) -> (x, check (lf - 1) xs lr r)
   {-# INLINE _Cons #-}
 
-instance (Choice p, Applicative f) => Snoc p f (Deque a) (Deque b) a b where
+instance Snoc (Deque a) (Deque b) a b where
   _Snoc = prism (\(BD lf f lr r,x) -> check lf f (lr + 1) (x : r)) $ \ (BD lf f lr r) ->
     if lf + lr == 0
     then Left empty
