@@ -1,11 +1,14 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE UndecidableInstances #-}
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.Fold
--- Copyright   :  (C) 2012-2013 Edward Kmett
+-- Copyright   :  (C) 2012-2014 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -22,6 +25,8 @@ module Control.Lens.Internal.Fold
   , Min(..), getMin
   , Leftmost(..), getLeftmost
   , Rightmost(..), getRightmost
+  , ReifiedMonoid(..), M(..)
+  , reifyFold
   ) where
 
 import Control.Applicative
@@ -30,6 +35,9 @@ import Data.Functor.Bind
 import Data.Functor.Contravariant
 import Data.Maybe
 import Data.Semigroup hiding (Min, getMin, Max, getMax)
+import Data.Reflection
+
+{-# ANN module "HLint: ignore Avoid lambda" #-}
 
 ------------------------------------------------------------------------------
 -- Folding
@@ -201,3 +209,24 @@ getRightmost :: Rightmost a -> Maybe a
 getRightmost RPure = Nothing
 getRightmost (RLeaf a) = Just a
 getRightmost (RStep x) = getRightmost x
+
+------------------------------------------------------------------------------
+-- Folding with Reified Monoid
+------------------------------------------------------------------------------
+
+data ReifiedMonoid a = ReifiedMonoid { reifiedMappend :: a -> a -> a, reifiedMempty :: a }
+
+instance Reifies s (ReifiedMonoid a) => Monoid (M a s) where
+  mappend (M x) (M y) = reflectResult (\m -> M (reifiedMappend m x y))
+  mempty              = reflectResult (\m -> M (reifiedMempty  m    ))
+
+reflectResult :: Reifies s a => (a -> f s) -> f s
+reflectResult f = let r = f (reflect r) in r
+
+newtype M a s = M a
+
+unM :: M a s -> proxy s -> a
+unM (M a) _ = a
+
+reifyFold :: (a -> a -> a) -> a -> (forall s. Reifies s (ReifiedMonoid a) => t -> M a s) -> t -> a
+reifyFold f z m xs = reify (ReifiedMonoid f z) (unM (m xs))
