@@ -15,7 +15,7 @@
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Lens
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -72,7 +72,9 @@ module Control.Lens.Lens
   , (%%@~), (%%@=)
   , (<%@~), (<%@=)
   , (<<%@~), (<<%@=)
-
+  -- ** General Purpose Combinators
+  , (&), (<&>), (??)
+  , (&~)
   -- * Lateral Composition
   , choosing
   , chosen
@@ -83,13 +85,17 @@ module Control.Lens.Lens
   , (<%~), (<+~), (<-~), (<*~), (<//~)
   , (<^~), (<^^~), (<**~)
   , (<||~), (<&&~), (<<>~)
-  , (<<%~), (<<.~)
+  , (<<%~), (<<.~), (<<+~), (<<-~), (<<*~)
+  , (<<//~), (<<^~), (<<^^~), (<<**~)
+  , (<<||~), (<<&&~), (<<<>~)
 
   -- * Setting State with Passthrough
   , (<%=), (<+=), (<-=), (<*=), (<//=)
   , (<^=), (<^^=), (<**=)
   , (<||=), (<&&=), (<<>=)
-  , (<<%=), (<<.=)
+  , (<<%=), (<<.=), (<<+=), (<<-=), (<<*=)
+  , (<<//=), (<<^=), (<<^^=), (<<**=)
+  , (<<||=), (<<&&=), (<<<>=)
   , (<<~)
 
   -- * Cloning Lenses
@@ -97,6 +103,9 @@ module Control.Lens.Lens
   , cloneIndexPreservingLens
   , cloneIndexedLens
 
+  -- * Arrow operators
+  , overA
+
   -- * ALens Combinators
   , storing
   , (^#)
@@ -114,9 +123,10 @@ module Control.Lens.Lens
   ) where
 
 import Control.Applicative
+import Control.Arrow
 import Control.Comonad
-import Control.Lens.Combinators
 import Control.Lens.Internal.Context
+import Control.Lens.Internal.Getter
 import Control.Lens.Internal.Indexed
 import Control.Lens.Type
 import Control.Monad.State as State
@@ -126,11 +136,14 @@ import Data.Profunctor.Rep
 import Data.Profunctor.Unsafe
 import Data.Void
 
+#ifdef HLINT
 {-# ANN module "HLint: ignore Use ***" #-}
+#endif
 
 -- $setup
 -- >>> :set -XNoOverloadedStrings
 -- >>> import Control.Lens
+-- >>> import Control.Monad.State
 -- >>> import Debug.SimpleReflect.Expr
 -- >>> import Debug.SimpleReflect.Vars as Vars hiding (f,g,h)
 -- >>> let f :: Expr -> Expr; f = Debug.SimpleReflect.Vars.f
@@ -143,6 +156,7 @@ infixl 8 ^#
 infixr 4 %%@~, <%@~, <<%@~, %%~, <+~, <*~, <-~, <//~, <^~, <^^~, <**~, <&&~, <||~, <<>~, <%~, <<%~, <<.~, <#~, #~, #%~, <#%~, #%%~
 infix  4 %%@=, <%@=, <<%@=, %%=, <+=, <*=, <-=, <//=, <^=, <^^=, <**=, <&&=, <||=, <<>=, <%=, <<%=, <<.=, <#=, #=, #%=, <#%=, #%%=
 infixr 2 <<~
+infixl 1 &, <&>, ??, &~
 
 -------------------------------------------------------------------------------
 -- Lenses
@@ -161,7 +175,7 @@ type ALens s t a b = LensLike (Pretext (->) a b) s t a b
 type ALens' s a = ALens s s a a
 
 -- | When you see this as an argument to a function, it expects an 'IndexedLens'
-type AnIndexedLens i s t a b = Overloading (Indexed i) (->) (Pretext (Indexed i) a b) s t a b
+type AnIndexedLens i s t a b = Optical (Indexed i) (->) (Pretext (Indexed i) a b) s t a b
 
 -- | @
 -- type 'AnIndexedLens'' = 'Simple' ('AnIndexedLens' i)
@@ -187,17 +201,16 @@ type AnIndexedLens' i s a  = AnIndexedLens i s s a a
 -- >>> s & lens getter setter %~ f
 -- setter s (f (getter s))
 --
+-- @
+-- 'lens' :: (s -> a) -> (s -> a -> s) -> 'Lens'' s a
+-- @
 lens :: (s -> a) -> (s -> b -> t) -> Lens s t a b
 lens sa sbt afb s = sbt s <$> afb (sa s)
 {-# INLINE lens #-}
 
--- | Build an index-preserving 'Lens' from a 'Control.Lens.Getter.Getter' and a 'Control.Lens.Setter.Setter'.
---
--- @
--- 'lens' :: (s -> a) -> (s -> b -> t) -> 'Lens' s t a b
--- 'lens' sa sbt afb s = sbt s '<$>' afb (sa s)
--- @
-iplens :: (Conjoined p, Functor f) => (s -> a) -> (s -> b -> t) -> Overloaded p f s t a b
+-- | Build an index-preserving 'Lens' from a 'Control.Lens.Getter.Getter' and a
+-- 'Control.Lens.Setter.Setter'.
+iplens :: (s -> a) -> (s -> b -> t) -> IndexPreservingLens s t a b
 iplens sa sbt pafb = cotabulate $ \ws -> sbt (extract ws) <$> corep pafb (sa <$> ws)
 {-# INLINE iplens #-}
 
@@ -207,6 +220,23 @@ ilens :: (s -> (i, a)) -> (s -> b -> t) -> IndexedLens i s t a b
 ilens sia sbt iafb s = sbt s <$> uncurry (indexed iafb) (sia s)
 {-# INLINE ilens #-}
 
+-- | This can be used to chain lens operations using @op=@ syntax
+-- rather than @op~@ syntax for simple non-type-changing cases.
+--
+-- >>> (10,20) & _1 .~ 30 & _2 .~ 40
+-- (30,40)
+--
+-- >>> (10,20) &~ do _1 .= 30; _2 .= 40
+-- (30,40)
+--
+-- This does not support type-changing assignment, /e.g./
+--
+-- >>> (10,20) & _1 .~ "hello"
+-- ("hello",20)
+(&~) :: s -> State s a -> s
+s &~ l = execState l s
+{-# INLINE (&~) #-}
+
 -- | ('%%~') can be used in one of two scenarios:
 --
 -- When applied to a 'Lens', it can edit the target of the 'Lens' in a
@@ -240,7 +270,7 @@ ilens sia sbt iafb s = sbt s <$> uncurry (indexed iafb) (sia s)
 -- ('%%~') ::             'Lens' s t a b      -> (a -> (r, b)) -> s -> (r, t)
 -- ('%%~') :: 'Monoid' m => 'Control.Lens.Traversal.Traversal' s t a b -> (a -> (m, b)) -> s -> (m, t)
 -- @
-(%%~) :: Overloading p q f s t a b -> p a (f b) -> q s (f t)
+(%%~) :: Optical p q f s t a b -> p a (f b) -> q s (f t)
 (%%~) = id
 {-# INLINE (%%~) #-}
 
@@ -264,7 +294,7 @@ ilens sia sbt iafb s = sbt s <$> uncurry (indexed iafb) (sia s)
 -- ('%%=') :: 'MonadState' s m             => 'Lens' s s a b      -> (a -> (r, b)) -> m r
 -- ('%%=') :: ('MonadState' s m, 'Monoid' r) => 'Control.Lens.Traversal.Traversal' s s a b -> (a -> (r, b)) -> m r
 -- @
-(%%=) :: MonadState s m => Overloading p (->) ((,) r) s s a b -> p a (r, b) -> m r
+(%%=) :: MonadState s m => Over p ((,) r) s s a b -> p a (r, b) -> m r
 #if MIN_VERSION_mtl(2,1,1)
 l %%= f = State.state (l f)
 #else
@@ -275,20 +305,80 @@ l %%= f = do
 #endif
 {-# INLINE (%%=) #-}
 
+-------------------------------------------------------------------------------
+-- General Purpose Combinators
+-------------------------------------------------------------------------------
+
+-- | Passes the result of the left side to the function on the right side (forward pipe operator).
+--
+-- This is the flipped version of ('$'), which is more common in languages like F# as (@|>@) where it is needed
+-- for inference. Here it is supplied for notational convenience and given a precedence that allows it
+-- to be nested inside uses of ('$').
+--
+-- >>> a & f
+-- f a
+--
+-- >>> "hello" & length & succ
+-- 6
+--
+-- This combinator is commonly used when applying multiple 'Lens' operations in sequence.
+--
+-- >>> ("hello","world") & _1.element 0 .~ 'j' & _1.element 4 .~ 'y'
+-- ("jelly","world")
+--
+-- This reads somewhat similar to:
+--
+-- >>> flip execState ("hello","world") $ do _1.element 0 .= 'j'; _1.element 4 .= 'y'
+-- ("jelly","world")
+(&) :: a -> (a -> b) -> b
+a & f = f a
+{-# INLINE (&) #-}
+
+-- | Infix flipped 'fmap'.
+--
+-- @
+-- ('<&>') = 'flip' 'fmap'
+-- @
+(<&>) :: Functor f => f a -> (a -> b) -> f b
+as <&> f = f <$> as
+{-# INLINE (<&>) #-}
+
+-- | This is convenient to 'flip' argument order of composite functions.
+--
+-- >>> over _2 ?? ("hello","world") $ length
+-- ("hello",5)
+--
+-- >>> over ?? length ?? ("hello","world") $ _2
+-- ("hello",5)
+(??) :: Functor f => f (a -> b) -> a -> f b
+fab ?? a = fmap ($ a) fab
+{-# INLINE (??) #-}
+
 -------------------------------------------------------------------------------
 -- Common Lenses
 -------------------------------------------------------------------------------
 
--- | Lift a 'Lens' so it can run under a function.
+-- | Lift a 'Lens' so it can run under a function (or other corepresentable profunctor).
 --
-inside :: ALens s t a b -> Lens (e -> s) (e -> t) (e -> a) (e -> b)
+-- @
+-- 'inside' :: 'Lens' s t a b -> 'Lens' (e -> s) (e -> t) (e -> a) (e -> b)
+-- @
+--
+inside :: Corepresentable p => ALens s t a b -> Lens (p e s) (p e t) (p e a) (p e b)
 inside l f es = o <$> f i where
-  i e = ipos (l sell (es e))
-  o ea e = ipeek (ea e) (l sell (es e))
---  i e = case l (Context id) (es e) of Context _ a -> a
---  o ea e = case l (Context id) (es e) of Context k _ -> k (ea e)
+  i = cotabulate $ \ e -> ipos $ l sell (corep es e)
+  o ea = cotabulate $ \ e -> ipeek (corep ea e) $ l sell (corep es e)
 {-# INLINE inside #-}
 
+{-
+-- | Lift a 'Lens' so it can run under a function (or any other corepresentable functor).
+insideF :: F.Representable f => ALens s t a b -> Lens (f s) (f t) (f a) (f b)
+insideF l f es = o <$> f i where
+  i = F.tabulate $ \e -> ipos $ l sell (F.index es e)
+  o ea = F.tabulate $ \ e -> ipeek (F.index ea e) $ l sell (F.index es e)
+{-# INLINE inside #-}
+-}
+
 -- | Merge two lenses, getters, setters, folds or traversals.
 --
 -- @
@@ -339,7 +429,8 @@ chosen pafb = cotabulate $ \weaa -> corep (either id id `lmap` pafb) weaa <&> \b
   Right _ -> Right b
 {-# INLINE chosen #-}
 
--- | 'alongside' makes a 'Lens' from two other lenses.
+-- | 'alongside' makes a 'Lens' from two other lenses or a 'Getter' from two other getters
+-- by executing them on their respective halves of a product.
 --
 -- >>> (Left a, Right b)^.alongside chosen chosen
 -- (a,b)
@@ -348,12 +439,16 @@ chosen pafb = cotabulate $ \weaa -> corep (either id id `lmap` pafb) weaa <&> \b
 -- (Left c,Right d)
 --
 -- @
--- 'alongside' :: 'Lens' s t a b -> 'Lens' s' t' a' b' -> 'Lens' (s,s') (t,t') (a,a') (b,b')
+-- 'alongside' :: 'Lens'   s t a b -> 'Lens'   s' t' a' b' -> 'Lens'   (s,s') (t,t') (a,a') (b,b')
+-- 'alongside' :: 'Getter' s t a b -> 'Getter' s' t' a' b' -> 'Getter' (s,s') (t,t') (a,a') (b,b')
 -- @
-alongside :: ALens s t a b -> ALens s' t' a' b' -> Lens (s,s') (t,t') (a,a') (b,b')
-alongside l r f (s, s') = f (ipos ls, ipos rs) <&> \(b, b') -> (ipeek b ls, ipeek b' rs) where
-  ls = l sell s
-  rs = r sell s'
+alongside :: LensLike (AlongsideLeft f b') s  t  a  b
+          -> LensLike (AlongsideRight f t) s' t' a' b'
+          -> LensLike f (s, s') (t, t') (a, a') (b, b')
+alongside l1 l2 f (a1, a2)
+  = getAlongsideRight $ l2 ?? a2 $ \b2 -> AlongsideRight
+  $ getAlongsideLeft  $ l1 ?? a1 $ \b1 -> AlongsideLeft
+  $ f (b1,b2)
 {-# INLINE alongside #-}
 
 -- | This 'Lens' lets you 'view' the current 'pos' of any indexed
@@ -415,7 +510,7 @@ cloneIndexedLens l f s = runPretext (l sell s) (Indexed (indexed f))
 -- ('<%~') ::             'Control.Lens.Iso.Iso' s t a b       -> (a -> b) -> s -> (b, t)
 -- ('<%~') :: 'Monoid' b => 'Control.Lens.Traversal.Traversal' s t a b -> (a -> b) -> s -> (b, t)
 -- @
-(<%~) :: Profunctor p => Overloading p q ((,) b) s t a b -> p a b -> q s (b, t)
+(<%~) :: Profunctor p => Optical p q ((,) b) s t a b -> p a b -> q s (b, t)
 l <%~ f = l $ rmap (\t -> (t, t)) f
 {-# INLINE (<%~) #-}
 
@@ -427,7 +522,7 @@ l <%~ f = l $ rmap (\t -> (t, t)) f
 -- ('<+~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<+~') :: 'Num' a => 'Control.Lens.Iso.Iso'' s a  -> a -> s -> (a, s)
 -- @
-(<+~) :: Num a => Overloading (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<+~) :: Num a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
 l <+~ a = l <%~ (+ a)
 {-# INLINE (<+~) #-}
 
@@ -439,7 +534,7 @@ l <+~ a = l <%~ (+ a)
 -- ('<-~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<-~') :: 'Num' a => 'Control.Lens.Iso.Iso'' s a  -> a -> s -> (a, s)
 -- @
-(<-~) :: Num a => Overloading (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<-~) :: Num a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
 l <-~ a = l <%~ subtract a
 {-# INLINE (<-~) #-}
 
@@ -452,7 +547,7 @@ l <-~ a = l <%~ subtract a
 -- ('<*~') :: 'Num' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<*~') :: 'Num' a => 'Control.Lens.Iso.Iso''  s a -> a -> s -> (a, s)
 -- @
-(<*~) :: Num a => Overloading (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<*~) :: Num a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
 l <*~ a = l <%~ (* a)
 {-# INLINE (<*~) #-}
 
@@ -464,46 +559,46 @@ l <*~ a = l <%~ (* a)
 -- ('<//~') :: 'Fractional' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<//~') :: 'Fractional' a => 'Control.Lens.Iso.Iso''  s a -> a -> s -> (a, s)
 -- @
-(<//~) :: Fractional a => Overloading (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<//~) :: Fractional a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
 l <//~ a = l <%~ (/ a)
 {-# INLINE (<//~) #-}
 
 -- | Raise the target of a numerically valued 'Lens' to a non-negative
 -- 'Integral' power and return the result.
 --
--- When you do not need the result of the division, ('Control.Lens.Setter.^~') is more flexible.
+-- When you do not need the result of the operation, ('Control.Lens.Setter.^~') is more flexible.
 --
 -- @
 -- ('<^~') :: ('Num' a, 'Integral' e) => 'Lens'' s a -> e -> s -> (a, s)
 -- ('<^~') :: ('Num' a, 'Integral' e) => 'Control.Lens.Iso.Iso'' s a -> e -> s -> (a, s)
 -- @
-(<^~) :: (Num a, Integral e) => Overloading (->) q ((,)a) s t a a -> e -> q s (a, t)
+(<^~) :: (Num a, Integral e) => Optical (->) q ((,)a) s t a a -> e -> q s (a, t)
 l <^~ e = l <%~ (^ e)
 {-# INLINE (<^~) #-}
 
 -- | Raise the target of a fractionally valued 'Lens' to an 'Integral' power
 -- and return the result.
 --
--- When you do not need the result of the division, ('Control.Lens.Setter.^^~') is more flexible.
+-- When you do not need the result of the operation, ('Control.Lens.Setter.^^~') is more flexible.
 --
 -- @
 -- ('<^^~') :: ('Fractional' a, 'Integral' e) => 'Lens'' s a -> e -> s -> (a, s)
 -- ('<^^~') :: ('Fractional' a, 'Integral' e) => 'Control.Lens.Iso.Iso'' s a -> e -> s -> (a, s)
 -- @
-(<^^~) :: (Fractional a, Integral e) => Overloading (->) q ((,)a) s t a a -> e -> q s (a, t)
+(<^^~) :: (Fractional a, Integral e) => Optical (->) q ((,)a) s t a a -> e -> q s (a, t)
 l <^^~ e = l <%~ (^^ e)
 {-# INLINE (<^^~) #-}
 
 -- | Raise the target of a floating-point valued 'Lens' to an arbitrary power
 -- and return the result.
 --
--- When you do not need the result of the division, ('Control.Lens.Setter.**~') is more flexible.
+-- When you do not need the result of the operation, ('Control.Lens.Setter.**~') is more flexible.
 --
 -- @
 -- ('<**~') :: 'Floating' a => 'Lens'' s a -> a -> s -> (a, s)
 -- ('<**~') :: 'Floating' a => 'Control.Lens.Iso.Iso'' s a  -> a -> s -> (a, s)
 -- @
-(<**~) :: Floating a => Overloading (->) q ((,)a) s t a a -> a -> q s (a, t)
+(<**~) :: Floating a => Optical (->) q ((,)a) s t a a -> a -> q s (a, t)
 l <**~ a = l <%~ (** a)
 {-# INLINE (<**~) #-}
 
@@ -515,7 +610,7 @@ l <**~ a = l <%~ (** a)
 -- ('<||~') :: 'Lens'' s 'Bool' -> 'Bool' -> s -> ('Bool', s)
 -- ('<||~') :: 'Control.Lens.Iso.Iso'' s 'Bool'  -> 'Bool' -> s -> ('Bool', s)
 -- @
-(<||~) :: Overloading (->) q ((,)Bool) s t Bool Bool -> Bool -> q s (Bool, t)
+(<||~) :: Optical (->) q ((,)Bool) s t Bool Bool -> Bool -> q s (Bool, t)
 l <||~ b = l <%~ (|| b)
 {-# INLINE (<||~) #-}
 
@@ -527,7 +622,7 @@ l <||~ b = l <%~ (|| b)
 -- ('<&&~') :: 'Lens'' s 'Bool' -> 'Bool' -> s -> ('Bool', s)
 -- ('<&&~') :: 'Control.Lens.Iso.Iso'' s 'Bool'  -> 'Bool' -> s -> ('Bool', s)
 -- @
-(<&&~) :: Overloading (->) q ((,)Bool) s t Bool Bool -> Bool -> q s (Bool, t)
+(<&&~) :: Optical (->) q ((,)Bool) s t Bool Bool -> Bool -> q s (Bool, t)
 l <&&~ b = l <%~ (&& b)
 {-# INLINE (<&&~) #-}
 
@@ -540,7 +635,7 @@ l <&&~ b = l <%~ (&& b)
 -- ('<<%~') ::             'Control.Lens.Iso.Iso' s t a b       -> (a -> b) -> s -> (a, t)
 -- ('<<%~') :: 'Monoid' a => 'Control.Lens.Traversal.Traversal' s t a b -> (a -> b) -> s -> (a, t)
 -- @
-(<<%~) :: Strong p => Overloading p q ((,)a) s t a b -> p a b -> q s (a, t)
+(<<%~) :: Strong p => Optical p q ((,)a) s t a b -> p a b -> q s (a, t)
 (<<%~) l = l . lmap (\a -> (a, a)) . second'
 {-# INLINE (<<%~) #-}
 
@@ -553,10 +648,50 @@ l <&&~ b = l <%~ (&& b)
 -- ('<<.~') ::             'Control.Lens.Iso.Iso' s t a b       -> b -> s -> (a, t)
 -- ('<<.~') :: 'Monoid' a => 'Control.Lens.Traversal.Traversal' s t a b -> b -> s -> (a, t)
 -- @
-(<<.~) :: Overloading (->) q ((,)a) s t a b -> b -> q s (a, t)
+(<<.~) :: Optical (->) q ((,)a) s t a b -> b -> q s (a, t)
 l <<.~ b = l $ \a -> (a, b)
 {-# INLINE (<<.~) #-}
 
+(<<+~) :: Num a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+l <<+~ b = l $ \a -> (a, a + b)
+{-# INLINE (<<+~) #-}
+
+(<<-~) :: Num a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+l <<-~ b = l $ \a -> (a, a - b)
+{-# INLINE (<<-~) #-}
+
+(<<*~) :: Num a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+l <<*~ b = l $ \a -> (a, a * b)
+{-# INLINE (<<*~) #-}
+
+(<<//~) :: Fractional a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+l <<//~ b = l $ \a -> (a, a / b)
+{-# INLINE (<<//~) #-}
+
+(<<^~) :: (Num a, Integral e) => Optical' (->) q ((,) a) s a -> e -> q s (a, s)
+l <<^~ e = l $ \a -> (a, a ^ e)
+{-# INLINE (<<^~) #-}
+
+(<<^^~) :: (Fractional a, Integral e) => Optical' (->) q ((,) a) s a -> e -> q s (a, s)
+l <<^^~ e = l $ \a -> (a, a ^^ e)
+{-# INLINE (<<^^~) #-}
+
+(<<**~) :: Floating a => Optical' (->) q ((,) a) s a -> a -> q s (a, s)
+l <<**~ e = l $ \a -> (a, a ** e)
+{-# INLINE (<<**~) #-}
+
+(<<||~) :: Optical' (->) q ((,) Bool) s Bool -> Bool -> q s (Bool, s)
+l <<||~ b = l $ \a -> (a, b || a)
+{-# INLINE (<<||~) #-}
+
+(<<&&~) :: Optical' (->) q ((,) Bool) s Bool -> Bool -> q s (Bool, s)
+l <<&&~ b = l $ \a -> (a, b && a)
+{-# INLINE (<<&&~) #-}
+
+(<<<>~) :: Monoid r => Optical' (->) q ((,) r) s r -> r -> q s (r, s)
+l <<<>~ b = l $ \a -> (a, a `mappend` b)
+{-# INLINE (<<<>~) #-}
+
 -------------------------------------------------------------------------------
 -- Setting and Remembering State
 -------------------------------------------------------------------------------
@@ -574,7 +709,7 @@ l <<.~ b = l $ \a -> (a, b)
 -- ('<%=') :: 'MonadState' s m             => 'Control.Lens.Iso.Iso'' s a       -> (a -> a) -> m a
 -- ('<%=') :: ('MonadState' s m, 'Monoid' a) => 'Control.Lens.Traversal.Traversal'' s a -> (a -> a) -> m a
 -- @
-(<%=) :: (Profunctor p, MonadState s m) => Overloading p (->) ((,)b) s s a b -> p a b -> m b
+(<%=) :: (Profunctor p, MonadState s m) => Over p ((,)b) s s a b -> p a b -> m b
 l <%= f = l %%= rmap (\b -> (b, b)) f
 {-# INLINE (<%=) #-}
 
@@ -637,7 +772,7 @@ l <//= a = l <%= (/ a)
 -- | Raise the target of a numerically valued 'Lens' into your 'Monad'\'s state
 -- to a non-negative 'Integral' power and return the result.
 --
--- When you do not need the result of the operation, ('Control.Lens.Setter.**=') is more flexible.
+-- When you do not need the result of the operation, ('Control.Lens.Setter.^=') is more flexible.
 --
 -- @
 -- ('<^=') :: ('MonadState' s m, 'Num' a, 'Integral' e) => 'Lens'' s a -> e -> m a
@@ -714,7 +849,7 @@ l <&&= b = l <%= (&& b)
 -- @
 --
 -- @('<<%=') :: 'MonadState' s m => 'LensLike' ((,)a) s s a b -> (a -> b) -> m a@
-(<<%=) :: (Strong p, MonadState s m) => Overloading p (->) ((,)a) s s a b -> p a b -> m a
+(<<%=) :: (Strong p, MonadState s m) => Over p ((,)a) s s a b -> p a b -> m a
 l <<%= f = l %%= lmap (\a -> (a,a)) (second' f)
 {-# INLINE (<<%=) #-}
 
@@ -724,17 +859,57 @@ l <<%= f = l %%= lmap (\a -> (a,a)) (second' f)
 -- When applied to a 'Control.Lens.Traversal.Traversal', it this will return a monoidal summary of all of the old values
 -- present.
 --
--- When you do not need the result of the operation, ('Control.Lens.Setter.%=') is more flexible.
+-- When you do not need the result of the operation, ('Control.Lens.Setter..=') is more flexible.
 --
 -- @
--- ('<<%=') :: 'MonadState' s m             => 'Lens'' s a      -> (a -> a) -> m a
--- ('<<%=') :: 'MonadState' s m             => 'Control.Lens.Iso.Iso'' s a       -> (a -> a) -> m a
--- ('<<%=') :: ('MonadState' s m, 'Monoid' t) => 'Control.Lens.Traversal.Traversal'' s a -> (a -> a) -> m a
+-- ('<<.=') :: 'MonadState' s m             => 'Lens'' s a      -> a -> m a
+-- ('<<.=') :: 'MonadState' s m             => 'Control.Lens.Iso.Iso'' s a       -> a -> m a
+-- ('<<.=') :: ('MonadState' s m, 'Monoid' t) => 'Control.Lens.Traversal.Traversal'' s a -> a -> m a
 -- @
 (<<.=) :: MonadState s m => LensLike ((,)a) s s a b -> b -> m a
 l <<.= b = l %%= \a -> (a,b)
 {-# INLINE (<<.=) #-}
 
+(<<+=) :: (MonadState s m, Num a) => LensLike' ((,) a) s a -> a -> m a
+l <<+= n = l %%= \a -> (a, a + n)
+{-# INLINE (<<+=) #-}
+
+(<<-=) :: (MonadState s m, Num a) => LensLike' ((,) a) s a -> a -> m a
+l <<-= n = l %%= \a -> (a, a - n)
+{-# INLINE (<<-=) #-}
+
+(<<*=) :: (MonadState s m, Num a) => LensLike' ((,) a) s a -> a -> m a
+l <<*= n = l %%= \a -> (a, a * n)
+{-# INLINE (<<*=) #-}
+
+(<<//=) :: (MonadState s m, Fractional a) => LensLike' ((,) a) s a -> a -> m a
+l <<//= n = l %%= \a -> (a, a / n)
+{-# INLINE (<<//=) #-}
+
+(<<^=) :: (MonadState s m, Num a, Integral e) => LensLike' ((,) a) s a -> e -> m a
+l <<^= n = l %%= \a -> (a, a ^ n)
+{-# INLINE (<<^=) #-}
+
+(<<^^=) :: (MonadState s m, Fractional a, Integral e) => LensLike' ((,) a) s a -> e -> m a
+l <<^^= n = l %%= \a -> (a, a ^^ n)
+{-# INLINE (<<^^=) #-}
+
+(<<**=) :: (MonadState s m, Floating a) => LensLike' ((,) a) s a -> a -> m a
+l <<**= n = l %%= \a -> (a, a ** n)
+{-# INLINE (<<**=) #-}
+
+(<<||=) :: MonadState s m => LensLike' ((,) Bool) s Bool -> Bool -> m Bool
+l <<||= b = l %%= \a -> (a, a || b)
+{-# INLINE (<<||=) #-}
+
+(<<&&=) :: MonadState s m => LensLike' ((,) Bool) s Bool -> Bool -> m Bool
+l <<&&= b = l %%= \a -> (a, a && b)
+{-# INLINE (<<&&=) #-}
+
+(<<<>=) :: (MonadState s m, Monoid r) => LensLike' ((,) r) s r -> r -> m r
+l <<<>= b = l %%= \a -> (a, a `mappend` b)
+{-# INLINE (<<<>=) #-}
+
 -- | Run a monadic action, and set the target of 'Lens' to its result.
 --
 -- @
@@ -755,7 +930,7 @@ l <<~ mb = do
 -- return the result.
 --
 -- When you do not need the result of the operation, ('Control.Lens.Setter.<>~') is more flexible.
-(<<>~) :: Monoid m => Overloading (->) q ((,)m) s t m m -> m -> q s (m, t)
+(<<>~) :: Monoid m => Optical (->) q ((,)m) s t m m -> m -> q s (m, t)
 l <<>~ m = l <%~ (`mappend` m)
 {-# INLINE (<<>~) #-}
 
@@ -767,6 +942,24 @@ l <<>~ m = l <%~ (`mappend` m)
 l <<>= r = l <%= (`mappend` r)
 {-# INLINE (<<>=) #-}
 
+------------------------------------------------------------------------------
+-- Arrow operators
+------------------------------------------------------------------------------
+
+-- | 'Control.Lens.Setter.over' for Arrows.
+--
+-- Unlike 'Control.Lens.Setter.over', 'overA' can't accept a simple
+-- 'Control.Lens.Setter.Setter', but requires a full lens, or close
+-- enough.
+--
+-- @
+-- overA :: Arrow ar => Lens s t a b -> ar a b -> ar s t
+-- @
+overA :: Arrow ar => LensLike (Context a b) s t a b -> ar a b -> ar s t
+overA l p = arr (\s -> let (Context f a) = l sell s in (f, a))
+            >>> second p
+            >>> arr (uncurry id)
+
 ------------------------------------------------------------------------------
 -- Indexed
 ------------------------------------------------------------------------------
@@ -785,7 +978,7 @@ l <<>= r = l <%= (`mappend` r)
 -- ('<%@~') ::             'IndexedLens' i s t a b      -> (i -> a -> b) -> s -> (b, t)
 -- ('<%@~') :: 'Monoid' b => 'Control.Lens.Traversal.IndexedTraversal' i s t a b -> (i -> a -> b) -> s -> (b, t)
 -- @
-(<%@~) :: Overloading (Indexed i) q ((,) b) s t a b -> (i -> a -> b) -> q s (b, t)
+(<%@~) :: Optical (Indexed i) q ((,) b) s t a b -> (i -> a -> b) -> q s (b, t)
 l <%@~ f = l (Indexed $ \i a -> let b = f i a in (b, b))
 {-# INLINE (<%@~) #-}
 
@@ -797,7 +990,7 @@ l <%@~ f = l (Indexed $ \i a -> let b = f i a in (b, b))
 -- ('<<%@~') ::             'IndexedLens' i s t a b      -> (i -> a -> b) -> s -> (a, t)
 -- ('<<%@~') :: 'Monoid' a => 'Control.Lens.Traversal.IndexedTraversal' i s t a b -> (i -> a -> b) -> s -> (a, t)
 -- @
-(<<%@~) :: Overloading (Indexed i) q ((,) a) s t a b -> (i -> a -> b) -> q s (a, t)
+(<<%@~) :: Optical (Indexed i) q ((,) a) s t a b -> (i -> a -> b) -> q s (a, t)
 l <<%@~ f = l $ Indexed $ \i a -> second' (f i) (a,a)
 
 {-# INLINE (<<%@~) #-}
@@ -964,7 +1157,7 @@ l #%%= f = do
 l <#~ b = \s -> (b, storing l b s)
 {-# INLINE (<#~) #-}
 
--- | A version of ('Control.Lens.Setter.<#=') that works on 'ALens'.
+-- | A version of ('Control.Lens.Setter.<.=') that works on 'ALens'.
 (<#=) :: MonadState s m => ALens s s a b -> b -> m b
 l <#= b = do
   l #= b
