@@ -1,11 +1,10 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE DefaultSignatures #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-#ifdef DEFAULT_SIGNATURES
-{-# LANGUAGE DefaultSignatures #-}
-#endif
+
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-} -- template-haskell
 #endif
@@ -13,6 +12,10 @@
 #ifndef MIN_VERSION_template_haskell
 #define MIN_VERSION_template_haskell(x,y,z) 1
 #endif
+
+#ifndef MIN_VERSION_free
+#define MIN_VERSION_free(x,y,z) 1
+#endif
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Plated
@@ -69,6 +72,7 @@ module Control.Lens.Plated
   , contexts, contextsOf, contextsOn, contextsOnOf
   , holes, holesOn, holesOnOf
   , para, paraOf
+  , (...)
 
   -- * Compos
   -- $compos
@@ -79,23 +83,32 @@ module Control.Lens.Plated
   )
   where
 
-import           Control.Applicative
-import           Control.Lens.Fold
-import           Control.Lens.Getter
-import           Control.Lens.Indexed
-import           Control.Lens.Internal.Context
-import           Control.Lens.Type
-import           Control.Lens.Setter
-import           Control.Lens.Traversal
-import qualified Language.Haskell.TH as TH
-#ifdef DEFAULT_SIGNATURES
-import           Data.Data
+import Control.Applicative
+import Control.Comonad.Cofree
+import Control.Lens.Fold
+import Control.Lens.Getter
+import Control.Lens.Indexed
+import Control.Lens.Internal.Context
+import Control.Lens.Type
+import Control.Lens.Setter
+import Control.Lens.Traversal
+import Control.Monad.Free as Monad
+import Control.Monad.Free.Church as Church
+import Control.Monad.Trans.Free as Trans
+#if !(MIN_VERSION_free(4,6,0))
+import Control.MonadPlus.Free as MonadPlus
 #endif
-import           Data.Data.Lens
-import           Data.Monoid
-import           Data.Tree
+import qualified Language.Haskell.TH as TH
+import Data.Aeson
+import Data.Bitraversable
+import Data.Data
+import Data.Data.Lens
+import Data.Monoid
+import Data.Tree
 
+#ifdef HLINT
 {-# ANN module "HLint: ignore Reduce duplication" #-}
+#endif
 
 -- | A 'Plated' type is one where we know how to extract its immediate self-similar children.
 --
@@ -192,7 +205,7 @@ class Plated a where
   -- 'plate' will default to 'uniplate' and you can choose to not override
   -- it with your own definition.
   plate :: Traversal' a a
-#ifdef DEFAULT_SIGNATURES
+#ifndef HLINT
   default plate :: Data a => Traversal' a a
   plate = uniplate
 #endif
@@ -201,18 +214,57 @@ instance Plated [a] where
   plate f (x:xs) = (x:) <$> f xs
   plate _ [] = pure []
 
+instance Traversable f => Plated (Monad.Free f a) where
+  plate f (Monad.Free as) = Monad.Free <$> traverse f as
+  plate _ x         = pure x
+
+instance (Traversable f, Traversable m) => Plated (Trans.FreeT f m a) where
+  plate f (Trans.FreeT xs) = Trans.FreeT <$> traverse (bitraverse pure f) xs
+
+#if !(MIN_VERSION_free(4,6,0))
+instance Traversable f => Plated (MonadPlus.Free f a) where
+  plate f (MonadPlus.Free as) = MonadPlus.Free <$> traverse f as
+  plate f (MonadPlus.Plus as) = MonadPlus.Plus <$> traverse f as
+  plate _ x         = pure x
+#endif
+
+instance Traversable f => Plated (Church.F f a) where
+  plate f = fmap Church.toF . plate (fmap Church.fromF . f . Church.toF) . Church.fromF
+
+-- -- This one can't work
+--
+-- instance (Traversable f, Traversable m) => Plated (ChurchT.FT f m a) where
+--   plate f = fmap ChurchT.toFT . plate (fmap ChurchT.fromFT . f . ChurchT.toFT) . ChurchT.fromFT
+
+instance Traversable f => Plated (Cofree f a) where
+  plate f (a :< as) = (:<) a <$> traverse f as
+
 instance Plated (Tree a) where
   plate f (Node a as) = Node a <$> traverse f as
 
-instance Plated TH.Exp where plate = uniplate
-instance Plated TH.Dec where plate = uniplate
-instance Plated TH.Con where plate = uniplate
-instance Plated TH.Type where plate = uniplate
+instance Plated Value where
+  plate f (Object o) = Object <$> traverse f o
+  plate f (Array a) = Array <$> traverse f a
+  plate _ xs = pure xs
+  {-# INLINE plate #-}
+
+{- Default uniplate instances -}
+instance Plated TH.Exp
+instance Plated TH.Dec
+instance Plated TH.Con
+instance Plated TH.Type
 #if !(MIN_VERSION_template_haskell(2,8,0))
-instance Plated TH.Kind where plate = uniplate -- in 2.8 Kind is an alias for Type
+instance Plated TH.Kind -- in 2.8 Kind is an alias for Type
 #endif
-instance Plated TH.Stmt where plate = uniplate
-instance Plated TH.Pat where plate = uniplate
+instance Plated TH.Stmt
+instance Plated TH.Pat
+
+
+infixr 9 ...
+-- | Compose through a plate
+(...) :: (Applicative f, Plated c) => LensLike f s t c c -> Over p f c c a b -> Over p f s t a b
+l ... m = l . plate . m
+{-# INLINE (...) #-}
 
 -------------------------------------------------------------------------------
 -- Children
@@ -525,7 +577,7 @@ holes = holesOf plate
 -- 'holesOn' :: 'IndexedLens'' i s a      -> s -> ['Pretext' ('Control.Lens.Internal.Indexed.Indexed' i) a a s]
 -- 'holesOn' :: 'IndexedTraversal'' i s a -> s -> ['Pretext' ('Control.Lens.Internal.Indexed.Indexed' i) a a s]
 -- @
-holesOn :: Conjoined p => Overloading p (->) (Bazaar p a a) s t a a -> s -> [Pretext p a a t]
+holesOn :: Conjoined p => Optical p (->) (Bazaar p a a) s t a a -> s -> [Pretext p a a t]
 holesOn = holesOf
 {-# INLINE holesOn #-}
 
@@ -544,7 +596,7 @@ holesOn = holesOf
 -- @
 holesOnOf :: Conjoined p
           => LensLike (Bazaar p  r r) s t a b
-          -> Overloading p (->) (Bazaar p r r) a b r r
+          -> Optical p (->) (Bazaar p r r) a b r r
           -> s -> [Pretext p r r t]
 holesOnOf b l = holesOf (b . l)
 {-# INLINE holesOnOf #-}
