@@ -1,12 +1,13 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE Rank2Types #-}
+
 #ifdef TRUSTWORTHY
 {-# LANGUAGE Trustworthy #-}
 #endif
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Prism
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -26,6 +27,7 @@ module Control.Lens.Prism
   , outside
   , aside
   , without
+  , below
   , isn't
   -- * Common Prisms
   , _Left
@@ -33,19 +35,23 @@ module Control.Lens.Prism
   , _Just
   , _Nothing
   , _Void
+  , _Show
   , only
+  , nearly
   -- * Prismatic profunctors
   , Choice(..)
   ) where
 
 import Control.Applicative
-import Control.Lens.Combinators
 import Control.Lens.Internal.Prism
-import Control.Lens.Internal.Setter
+import Control.Lens.Lens
+import Control.Lens.Review
 import Control.Lens.Type
 import Control.Monad
-import Data.Bifunctor
+import Data.Functor.Identity
 import Data.Profunctor
+import Data.Profunctor.Rep
+import Data.Traversable
 import Data.Void
 #ifndef SAFE
 import Unsafe.Coerce
@@ -71,7 +77,7 @@ import Data.Profunctor.Unsafe
 ------------------------------------------------------------------------------
 
 -- | If you see this in a signature for a function, the function is expecting a 'Prism'.
-type APrism s t a b = Market a b a (Mutator b) -> Market a b s (Mutator t)
+type APrism s t a b = Market a b a (Identity b) -> Market a b s (Identity t)
 
 -- | @
 -- type APrism' = 'Simple' 'APrism'
@@ -81,10 +87,10 @@ type APrism' s a = APrism s s a a
 -- | Convert 'APrism' to the pair of functions that characterize it.
 runPrism :: APrism s t a b -> Market a b s t
 #ifdef SAFE
-runPrism k = case k (Market Mutator Right) of
-  Market bt seta -> Market (runMutator #. bt) (either (Left . runMutator) Right . seta)
+runPrism k = case k (Market Identity Right) of
+  Market bt seta -> Market (runIdentity #. bt) (either (Left . runIdentity) Right . seta)
 #else
-runPrism k = unsafeCoerce (k (Market Mutator Right))
+runPrism k = unsafeCoerce (k (Market Identity Right))
 #endif
 {-# INLINE runPrism #-}
 
@@ -117,18 +123,12 @@ prism' bs sma = prism bs (\s -> maybe (Left s) Right (sma s))
 -- | Use a 'Prism' as a kind of first-class pattern.
 --
 -- @'outside' :: 'Prism' s t a b -> 'Lens' (t -> r) (s -> r) (b -> r) (a -> r)@
-outside :: APrism s t a b -> Lens (t -> r) (s -> r) (b -> r) (a -> r)
-outside k = case runPrism k of
-  Market bt seta -> \f tr -> f (tr.bt) <&> \ar -> either tr ar . seta
-{-# INLINE outside #-}
 
--- | Use a 'Prism' to work over part of a structure.
-aside :: APrism s t a b -> Prism (e, s) (e, t) (e, a) (e, b)
-aside k = case runPrism k of
-  Market bt seta -> prism (fmap bt) $ \(e,s) -> case seta s of
-    Left t -> Left (e,t)
-    Right a -> Right (e,a)
-{-# INLINE aside #-}
+-- TODO: can we make this work with merely Strong?
+outside :: Representable p => APrism s t a b -> Lens (p t r) (p s r) (p b r) (p a r)
+outside k = case runPrism k of
+  Market bt seta -> \f ft -> f (lmap bt ft) <&> \fa -> tabulate $ either (rep ft) (rep fa) . seta
+{-# INLINE outside #-}
 
 -- | Given a pair of prisms, project sums.
 --
@@ -143,6 +143,23 @@ without k = case runPrism k of
       Right u -> bimap Right Right (uevc u)
 {-# INLINE without #-}
 
+-- | Use a 'Prism' to work over part of a structure.
+--
+aside :: APrism s t a b -> Prism (e, s) (e, t) (e, a) (e, b)
+aside k = case runPrism k of
+  Market bt seta -> prism (fmap bt) $ \(e,s) -> case seta s of
+    Left t  -> Left  (e,t)
+    Right a -> Right (e,a)
+{-# INLINE aside #-}
+
+-- | 'lift' a 'Prism' through a 'Traversable' functor, giving a Prism that matches only if all the elements of the container match the 'Prism'.
+below :: Traversable f => APrism' s a -> Prism' (f s) (f a)
+below k = case runPrism k of
+  Market bt seta -> prism (fmap bt) $ \s -> case traverse seta s of
+    Left _  -> Left s
+    Right t -> Right t
+{-# INLINE below #-}
+
 -- | Check to see if this 'Prism' doesn't match.
 --
 -- >>> isn't _Left (Right 12)
@@ -150,6 +167,9 @@ without k = case runPrism k of
 --
 -- >>> isn't _Left (Left 12)
 -- False
+--
+-- >>> isn't _Empty []
+-- False
 isn't :: APrism s t a b -> s -> Bool
 isn't k s = case runPrism k of
   Market _ seta -> case seta s of
@@ -272,3 +292,36 @@ _Void = prism absurd Left
 only :: Eq a => a -> Prism' a ()
 only a = prism' (\() -> a) $ guard . (a ==)
 {-# INLINE only #-}
+
+
+-- | This 'Prism' compares for approximate equality with a given value and a predicate for testing.
+--
+-- To comply with the 'Prism' laws the arguments you supply to @nearly a p@ are somewhat constrained.
+--
+-- We assume @p x@ holds iff @x ≡ a@. Under that assumption then this is a valid 'Prism'.
+--
+-- This is useful when working with a type where you can test equality for only a subset of its
+-- values, and the prism selects such a value.
+nearly :: a -> (a -> Bool) -> Prism' a ()
+nearly a p = prism' (\() -> a) $ guard . p
+{-# INLINE nearly #-}
+
+-- | This is an improper prism for text formatting based on 'Read' and 'Show'.
+--
+-- This 'Prism' is \"improper\" in the sense that it normalizes the text formatting, but round tripping
+-- is idempotent given sane 'Read'/'Show' instances.
+--
+-- >>> _Show # 2
+-- "2"
+--
+-- >>> "EQ" ^? _Show :: Maybe Ordering
+-- Just EQ
+--
+-- @
+-- '_Show' ≡ 'prism'' 'show' 'readMaybe'
+-- @
+_Show :: (Read a, Show a) => Prism' String a
+_Show = prism show $ \s -> case reads s of
+  [(a,"")] -> Right a
+  _ -> Left s
+{-# INLINE _Show #-}
