@@ -6,7 +6,7 @@
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Review
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -21,6 +21,7 @@ module Control.Lens.Review
     Review, Review'
   , AReview, AReview'
   , unto
+  , un
   , re
   , review, reviews
   , reuse, reuses
@@ -40,6 +41,7 @@ import Data.Bifunctor
 import Data.Functor.Identity
 import Data.Profunctor
 import Data.Profunctor.Unsafe
+import Data.Tagged
 import Data.Void
 
 -- $setup
@@ -61,14 +63,14 @@ infixr 8 #
 --
 -- You can generate a 'Review' by using 'unto'. You can also use any 'Prism' or 'Iso'
 -- directly as a 'Review'.
-type Review s t a b = forall p f. (Profunctor p, Bifunctor p, Settable f) => Overloaded p f s t a b
+type Review s t a b = forall p f. (Profunctor p, Bifunctor p, Settable f) => Optic p f s t a b
 
 -- | A 'Simple' 'Review'
 type Review' t b = Review t t b b
 
 -- | If you see this in a signature for a function, the function is expecting a 'Review'
 -- (in practice, this usually means a 'Prism').
-type AReview s t a b = Overloaded Reviewed Identity s t a b
+type AReview s t a b = Optic Tagged Identity s t a b
 
 -- | A 'Simple' 'AReview'
 type AReview' t b = AReview t t b b
@@ -78,10 +80,26 @@ type AReview' t b = AReview t t b b
 -- @
 -- 'unto' :: (b -> t) -> 'Review'' t b
 -- @
-unto :: (Profunctor p, Bifunctor p, Functor f) => (b -> t) -> Overloaded p f s t a b
+--
+-- @
+-- 'unto' = 'un' . 'to'
+-- @
+unto :: (Profunctor p, Bifunctor p, Functor f) => (b -> t) -> Optic p f s t a b
 unto f = first absurd . lmap absurd . rmap (fmap f)
 {-# INLINE unto #-}
 
+-- | Turn a 'Getter' around to get a 'Review'
+--
+-- @
+-- 'un' = 'unto' . 'view'
+-- 'unto' = 'un' . 'to'
+-- @
+--
+-- >>> un (to length) # [1,2,3]
+-- 3
+un :: (Profunctor p, Bifunctor p, Functor f) => Getting a s a -> Optic' p f a s
+un = unto . view
+
 -- | Turn a 'Prism' or 'Control.Lens.Iso.Iso' around to build a 'Getter'.
 --
 -- If you have an 'Control.Lens.Iso.Iso', 'Control.Lens.Iso.from' is a more powerful version of this function
@@ -105,7 +123,7 @@ unto f = first absurd . lmap absurd . rmap (fmap f)
 -- 're' :: 'Iso' s t a b   -> 'Getter' b t
 -- @
 re :: AReview s t a b -> Getter b t
-re p = to (runIdentity #. runReviewed #. p .# Reviewed .# Identity)
+re p = to (runIdentity #. unTagged #. p .# Tagged .# Identity)
 {-# INLINE re #-}
 
 -- | This can be used to turn an 'Control.Lens.Iso.Iso' or 'Prism' around and 'view' a value (or the current environment) through it the other way.
@@ -137,7 +155,7 @@ re p = to (runIdentity #. runReviewed #. p .# Reviewed .# Identity)
 -- 'review' :: 'MonadReader' a m => 'Prism'' s a -> m s
 -- @
 review :: MonadReader b m => AReview s t a b -> m t
-review p = asks (runIdentity #. runReviewed #. p .# Reviewed .# Identity)
+review p = asks (runIdentity #. unTagged #. p .# Tagged .# Identity)
 {-# INLINE review #-}
 
 -- | An infix alias for 'review'.
@@ -164,7 +182,7 @@ review p = asks (runIdentity #. runReviewed #. p .# Reviewed .# Identity)
 -- (#) :: 'Equality'' s a -> a -> s
 -- @
 ( # ) :: AReview s t a b -> b -> t
-( # ) p = runIdentity #. runReviewed #. p .# Reviewed .# Identity
+( # ) p = runIdentity #. unTagged #. p .# Tagged .# Identity
 {-# INLINE ( # ) #-}
 
 -- | This can be used to turn an 'Control.Lens.Iso.Iso' or 'Prism' around and 'view' a value (or the current environment) through it the other way,
@@ -197,7 +215,7 @@ review p = asks (runIdentity #. runReviewed #. p .# Reviewed .# Identity)
 -- 'reviews' :: 'MonadReader' a m => 'Prism'' s a -> (s -> r) -> m r
 -- @
 reviews :: MonadReader b m => AReview s t a b -> (t -> r) -> m r
-reviews p tr = asks (tr . runIdentity #. runReviewed #. p .# Reviewed .# Identity)
+reviews p tr = asks (tr . runIdentity #. unTagged #. p .# Tagged .# Identity)
 {-# INLINE reviews #-}
 
 -- | This can be used to turn an 'Control.Lens.Iso.Iso' or 'Prism' around and 'use' a value (or the current environment) through it the other way.
@@ -218,7 +236,7 @@ reviews p tr = asks (tr . runIdentity #. runReviewed #. p .# Reviewed .# Identit
 -- 'reuse' :: 'MonadState' a m => 'Iso'' s a   -> m s
 -- @
 reuse :: MonadState b m => AReview s t a b -> m t
-reuse p = gets (runIdentity #. runReviewed #. p .# Reviewed .# Identity)
+reuse p = gets (runIdentity #. unTagged #. p .# Tagged .# Identity)
 {-# INLINE reuse #-}
 
 -- | This can be used to turn an 'Control.Lens.Iso.Iso' or 'Prism' around and 'use' the current state through it the other way,
@@ -237,5 +255,5 @@ reuse p = gets (runIdentity #. runReviewed #. p .# Reviewed .# Identity)
 -- 'reuses' :: 'MonadState' a m => 'Iso'' s a   -> (s -> r) -> m r
 -- @
 reuses :: MonadState b m => AReview s t a b -> (t -> r) -> m r
-reuses p tr = gets (tr . runIdentity #. runReviewed #. p .# Reviewed .# Identity)
+reuses p tr = gets (tr . runIdentity #. unTagged #. p .# Tagged .# Identity)
 {-# INLINE reuses #-}
