@@ -9,7 +9,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Setter
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -66,14 +66,17 @@ module Control.Lens.Setter
   , imapOf, iover
   , isets
   , (%@~), (%@=)
+  -- * Arrow operators
+  , assignA
   -- * Exported for legible error messages
   , Settable
-  , Mutator
+  , Identity(..)
   -- * Deprecated
   , mapOf
   ) where
 
 import Control.Applicative
+import Control.Arrow
 import Control.Comonad
 import Control.Lens.Internal.Indexed
 import Control.Lens.Internal.Setter
@@ -82,12 +85,15 @@ import Control.Monad (liftM)
 import Control.Monad.State.Class as State
 import Control.Monad.Writer.Class as Writer
 import Data.Functor.Contravariant
+import Data.Functor.Identity
 import Data.Monoid
 import Data.Profunctor
 import Data.Profunctor.Rep
 import Data.Profunctor.Unsafe
 
+#ifdef HLINT
 {-# ANN module "HLint: ignore Avoid lambda" #-}
+#endif
 
 -- $setup
 -- >>> import Control.Lens
@@ -115,9 +121,7 @@ infixr 2 <~
 --
 -- When consuming a setter directly to perform a mapping, you can use this type, but most
 -- user code will not need to use this type.
---
--- By choosing 'Mutator' rather than 'Data.Functor.Identity.Identity', we get nicer error messages.
-type ASetter s t a b = (a -> Mutator b) -> s -> Mutator t
+type ASetter s t a b = (a -> Identity b) -> s -> Identity t
 
 -- | This is a useful alias for use when consuming a 'Setter''.
 --
@@ -132,9 +136,7 @@ type ASetter' s a = ASetter s s a a
 --
 -- When consuming a setter directly to perform a mapping, you can use this type, but most
 -- user code will not need to use this type.
---
--- By choosing 'Mutator' rather than 'Data.Functor.Identity.Identity', we get nicer error messages.
-type AnIndexedSetter i s t a b = Indexed i a (Mutator b) -> s -> Mutator t
+type AnIndexedSetter i s t a b = Indexed i a (Identity b) -> s -> Identity t
 
 -- | @
 -- type 'AnIndexedSetter'' i = 'Simple' ('AnIndexedSetter' i)
@@ -144,7 +146,7 @@ type AnIndexedSetter' i s a = AnIndexedSetter i s s a a
 -- | This is a convenient alias when defining highly polymorphic code that takes both
 -- 'ASetter' and 'AnIndexedSetter' as appropriate. If a function takes this it is
 -- expecting one of those two things based on context.
-type Setting p s t a b = p a (Mutator b) -> s -> Mutator t
+type Setting p s t a b = p a (Identity b) -> s -> Identity t
 
 -- | This is a convenient alias when defining highly polymorphic code that takes both
 -- 'ASetter'' and 'AnIndexedSetter'' as appropriate. If a function takes this it is
@@ -219,7 +221,7 @@ lifted = sets liftM
 -- >>> getOp (over contramapped (*5) (Op show)) 100
 -- "500"
 --
--- >>> Prelude.map ($ 1) $ over (mapped . wrapping Op . contramapped) (*12) [(*2),(+1),(^3)]
+-- >>> Prelude.map ($ 1) $ over (mapped . _Unwrapping' Op . contramapped) (*12) [(*2),(+1),(^3)]
 -- [24,13,1728]
 --
 contramapped :: Contravariant f => Setter (f b) (f a) a b
@@ -283,24 +285,24 @@ setting l pafb = cotabulate $ \ws -> pure $ l (\a -> untainted (corep pafb (a <$
 -- @
 -- 'sets' :: ((a -> b) -> s -> t) -> 'Setter' s t a b
 -- @
-sets :: (Profunctor p, Profunctor q, Settable f) => (p a b -> q s t) -> Overloading p q f s t a b
+sets :: (Profunctor p, Profunctor q, Settable f) => (p a b -> q s t) -> Optical p q f s t a b
 sets f g = taintedDot (f (untaintedDot g))
 {-# INLINE sets #-}
 
 -- | Restore 'ASetter' to a full 'Setter'.
 cloneSetter :: ASetter s t a b -> Setter s t a b
-cloneSetter l afb = taintedDot $ runMutator #. l (Mutator #. untaintedDot afb)
+cloneSetter l afb = taintedDot $ runIdentity #. l (Identity #. untaintedDot afb)
 {-# INLINE cloneSetter #-}
 
 -- | Build an 'IndexPreservingSetter' from any 'Setter'.
 cloneIndexPreservingSetter :: ASetter s t a b -> IndexPreservingSetter s t a b
 cloneIndexPreservingSetter l pafb = cotabulate $ \ws ->
-    taintedDot runMutator $ l (\a -> Mutator (untainted (corep pafb (a <$ ws)))) (extract ws)
+    taintedDot runIdentity $ l (\a -> Identity (untainted (corep pafb (a <$ ws)))) (extract ws)
 {-# INLINE cloneIndexPreservingSetter #-}
 
 -- | Clone an 'IndexedSetter'.
 cloneIndexedSetter :: AnIndexedSetter i s t a b -> IndexedSetter i s t a b
-cloneIndexedSetter l pafb = taintedDot (runMutator #. l (Indexed $ \i -> Mutator #. untaintedDot (indexed pafb i)))
+cloneIndexedSetter l pafb = taintedDot (runIdentity #. l (Indexed $ \i -> Identity #. untaintedDot (indexed pafb i)))
 {-# INLINE cloneIndexedSetter #-}
 
 -----------------------------------------------------------------------------
@@ -348,7 +350,7 @@ cloneIndexedSetter l pafb = taintedDot (runMutator #. l (Indexed $ \i -> Mutator
 -- 'over' :: 'ASetter' s t a b -> (a -> b) -> s -> t
 -- @
 over :: Profunctor p => Setting p s t a b -> p a b -> s -> t
-over l f = runMutator #. l (Mutator #. f)
+over l f = runIdentity #. l (Identity #. f)
 {-# INLINE over #-}
 
 -- | Replace the target of a 'Lens' or all of the targets of a 'Setter'
@@ -374,7 +376,7 @@ over l f = runMutator #. l (Mutator #. f)
 -- 'set' :: 'Traversal' s t a b -> b -> s -> t
 -- @
 set :: ASetter s t a b -> b -> s -> t
-set l b = runMutator #. l (\_ -> Mutator b)
+set l b = runIdentity #. l (\_ -> Identity b)
 {-# INLINE set #-}
 
 -- | Replace the target of a 'Lens' or all of the targets of a 'Setter''
@@ -401,7 +403,7 @@ set l b = runMutator #. l (\_ -> Mutator b)
 -- 'set'' :: 'Traversal'' s a -> a -> s -> s
 -- @
 set' :: ASetter' s a -> a -> s -> s
-set' l b = runMutator #. l (\_ -> Mutator b)
+set' l b = runIdentity #. l (\_ -> Identity b)
 {-# INLINE set' #-}
 
 -- | Modifies the target of a 'Lens' or all of the targets of a 'Setter' or
@@ -669,8 +671,8 @@ l ^^~ n = over l (^^ n)
 -- >>> (a,b) & both **~ c
 -- (a**c,b**c)
 --
--- >>> _2 **~ pi $ (1,3)
--- (1,31.54428070019754)
+-- >>> _2 **~ 10 $ (3,2)
+-- (3,1024.0)
 --
 -- @
 -- ('**~') :: 'Floating' a => 'Setter'' s a    -> a -> s -> s
@@ -762,7 +764,7 @@ assign l b = State.modify (set l b)
 -- ('.=') :: 'MonadState' s m => 'Setter'' s a    -> a -> m ()
 -- @
 --
--- "It puts the state in the monad or it gets the hose again."
+-- /It puts the state in the monad or it gets the hose again./
 (.=) :: MonadState s m => ASetter s s a b -> b -> m ()
 l .= b = State.modify (l .~ b)
 {-# INLINE (.=) #-}
@@ -1185,6 +1187,41 @@ l %@~ f = l %~ Indexed f
 l %@= f = State.modify (l %@~ f)
 {-# INLINE (%@=) #-}
 
+------------------------------------------------------------------------------
+-- Arrows
+------------------------------------------------------------------------------
+
+-- | Run an arrow command and use the output to set all the targets of
+-- a 'Lens', 'Setter' or 'Traversal' to the result.
+--
+-- 'assignA' can be used very similarly to ('<~'), except that the type of
+-- the object being modified can change; for example:
+--
+-- @
+-- runKleisli action ((), (), ()) where
+--   action =      assignA _1 (Kleisli (const getVal1))
+--            \>>> assignA _2 (Kleisli (const getVal2))
+--            \>>> assignA _3 (Kleisli (const getVal3))
+--   getVal1 :: Either String Int
+--   getVal1 = ...
+--   getVal2 :: Either String Bool
+--   getVal2 = ...
+--   getVal3 :: Either String Char
+--   getVal3 = ...
+-- @
+--
+-- has the type @'Either' 'String' ('Int', 'Bool', 'Char')@
+--
+-- @
+-- 'assignA' :: 'Arrow' p => 'Iso' s t a b       -> p s b -> p s t
+-- 'assignA' :: 'Arrow' p => 'Lens' s t a b      -> p s b -> p s t
+-- 'assignA' :: 'Arrow' p => 'Traversal' s t a b -> p s b -> p s t
+-- 'assignA' :: 'Arrow' p => 'Setter' s t a b    -> p s b -> p s t
+-- @
+assignA :: Arrow p => ASetter s t a b -> p s b -> p s t
+assignA l p = arr (flip $ set l) &&& p >>> arr (uncurry id)
+{-# INLINE assignA #-}
+
 ------------------------------------------------------------------------------
 -- Deprecated
 ------------------------------------------------------------------------------
