@@ -17,7 +17,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Traversal
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -48,10 +48,15 @@ module Control.Lens.Traversal
   (
   -- * Traversals
     Traversal, Traversal'
+  , Traversal1, Traversal1'
   , IndexedTraversal, IndexedTraversal'
+  , IndexedTraversal1, IndexedTraversal1'
   , ATraversal, ATraversal'
+  , ATraversal1, ATraversal1'
   , AnIndexedTraversal, AnIndexedTraversal'
+  , AnIndexedTraversal1, AnIndexedTraversal1'
   , Traversing, Traversing'
+  , Traversing1, Traversing1'
 
   -- * Traversing and Lensing
   , traverseOf, forOf, sequenceAOf
@@ -59,12 +64,15 @@ module Control.Lens.Traversal
   , transposeOf
   , mapAccumLOf, mapAccumROf
   , scanr1Of, scanl1Of
-  , failover
+  , failover, ifailover
 
   -- * Monomorphic Traversals
   , cloneTraversal
   , cloneIndexPreservingTraversal
   , cloneIndexedTraversal
+  , cloneTraversal1
+  , cloneIndexPreservingTraversal1
+  , cloneIndexedTraversal1
 
   -- * Parts and Holes
   , partsOf, partsOf'
@@ -74,10 +82,12 @@ module Control.Lens.Traversal
 
   -- * Common Traversals
   , Traversable(traverse)
+  , Traversable1(traverse1)
   , both
   , beside
   , taking
   , dropping
+  , failing
 
   -- * Indexed Traversals
 
@@ -86,6 +96,7 @@ module Control.Lens.Traversal
   , TraverseMin(..)
   , TraverseMax(..)
   , traversed
+  , traversed1
   , traversed64
   , elementOf
   , element
@@ -105,8 +116,8 @@ module Control.Lens.Traversal
   , imapAccumLOf
 
   -- * Implementation Details
-  , Bazaar(..)
-  , Bazaar'
+  , Bazaar(..), Bazaar'
+  , Bazaar1(..), Bazaar1'
   , loci
   , iloci
   ) where
@@ -115,25 +126,29 @@ import Control.Applicative as Applicative
 import Control.Applicative.Backwards
 import Control.Category
 import Control.Comonad
-import Control.Monad
-import Control.Lens.Combinators
 import Control.Lens.Fold
 import Control.Lens.Getter (coerced)
 import Control.Lens.Internal.Bazaar
 import Control.Lens.Internal.Context
 import Control.Lens.Internal.Indexed
+import Control.Lens.Lens
 import Control.Lens.Type
+import Control.Monad
 import Control.Monad.Trans.State.Lazy
+import Data.Bitraversable
 import Data.Functor.Compose
 import Data.Int
 import Data.IntMap as IntMap
 import Data.Map as Map
 import Data.Monoid
-import Data.Traversable
-import Data.Tuple (swap)
 import Data.Profunctor
 import Data.Profunctor.Rep
 import Data.Profunctor.Unsafe
+import Data.Semigroup.Traversable
+import Data.Tagged
+import Data.Traversable
+import Data.Tuple (swap)
+import GHC.Magic (inline)
 import Prelude hiding ((.),id)
 
 -- $setup
@@ -142,6 +157,7 @@ import Prelude hiding ((.),id)
 -- >>> import Control.DeepSeq (NFData (..), force)
 -- >>> import Control.Exception (evaluate)
 -- >>> import Data.Maybe (fromMaybe)
+-- >>> import Debug.SimpleReflect.Vars
 -- >>> import Data.Void
 -- >>> import System.Timeout (timeout)
 -- >>> let timingOut :: NFData a => a -> IO a; timingOut = fmap (fromMaybe (error "timeout")) . timeout (5*10^6) . evaluate . force
@@ -158,14 +174,32 @@ type ATraversal s t a b = LensLike (Bazaar (->) a b) s t a b
 -- @
 type ATraversal' s a = ATraversal s s a a
 
+
+-- | When you see this as an argument to a function, it expects a 'Traversal1'.
+type ATraversal1 s t a b = LensLike (Bazaar1 (->) a b) s t a b
+
+-- | @
+-- type 'ATraversal1'' = 'Simple' 'ATraversal1'
+-- @
+type ATraversal1' s a = ATraversal1 s s a a
+
 -- | When you see this as an argument to a function, it expects an 'IndexedTraversal'.
 type AnIndexedTraversal i s t a b = Over (Indexed i) (Bazaar (Indexed i) a b) s t a b
 
+-- | When you see this as an argument to a function, it expects an 'IndexedTraversal1'.
+type AnIndexedTraversal1 i s t a b = Over (Indexed i) (Bazaar1 (Indexed i) a b) s t a b
+
 -- | @
 -- type 'AnIndexedTraversal'' = 'Simple' ('AnIndexedTraversal' i)
 -- @
 type AnIndexedTraversal' i s a = AnIndexedTraversal i s s a a
 
+-- | @
+-- type 'AnIndexedTraversal1'' = 'Simple' ('AnIndexedTraversal1' i)
+-- @
+type AnIndexedTraversal1' i s a = AnIndexedTraversal1 i s s a a
+
+
 -- | When you see this as an argument to a function, it expects
 --
 --  * to be indexed if @p@ is an instance of 'Indexed' i,
@@ -181,10 +215,13 @@ type AnIndexedTraversal' i s a = AnIndexedTraversal i s s a a
 --  * a 'Fold' if 'f' is 'Gettable' and 'Applicative'.
 type Traversing p f s t a b = Over p (BazaarT p f a b) s t a b
 
+type Traversing1 p f s t a b = Over p (BazaarT1 p f a b) s t a b
+
 -- | @
 -- type 'Traversing'' f = 'Simple' ('Traversing' f)
 -- @
 type Traversing' p f s a = Traversing p f s s a a
+type Traversing1' p f s a = Traversing1 p f s s a a
 
 --------------------------
 -- Traversal Combinators
@@ -451,8 +488,17 @@ iloci f w = getCompose (runBazaar w (Compose #. Indexed (\i -> fmap (indexed sel
 --
 -- /Note:/ You should really try to maintain the invariant of the number of children in the list.
 --
+-- >>> (a,b,c) & partsOf each .~ [x,y,z]
+-- (x,y,z)
+--
 -- Any extras will be lost. If you do not supply enough, then the remainder will come from the original structure.
 --
+-- >>> (a,b,c) & partsOf each .~ [w,x,y,z]
+-- (w,x,y)
+--
+-- >>> (a,b,c) & partsOf each .~ [x,y]
+-- (x,y,c)
+--
 -- So technically, this is only a 'Lens' if you do not change the number of results it returns.
 --
 -- When applied to a 'Fold' the result is merely a 'Getter'.
@@ -470,9 +516,9 @@ partsOf l f s = outs b <$> f (ins b) where b = l sell s
 
 -- | An indexed version of 'partsOf' that receives the entire list of indices as its index.
 ipartsOf :: forall i p f s t a. (Indexable [i] p, Functor f) => Traversing (Indexed i) f s t a a -> Over p f s t [a] [a]
-ipartsOf l f s = outs b <$> indexed f (is :: [i]) as where
-  (is,as) = unzip (pins b)
-  b = l sell s
+ipartsOf l = conjoined
+  (\f s -> let b = inline l sell s                            in outs b <$> f (wins b))
+  (\f s -> let b = inline l sell s; (is, as) = unzip (pins b) in outs b <$> indexed f (is :: [i]) as)
 {-# INLINE ipartsOf #-}
 
 -- | A type-restricted version of 'partsOf' that can only be used with a 'Traversal'.
@@ -482,9 +528,9 @@ partsOf' l f s = outs b <$> f (ins b) where b = l sell s
 
 -- | A type-restricted version of 'ipartsOf' that can only be used with an 'IndexedTraversal'.
 ipartsOf' :: forall i p f s t a. (Indexable [i] p, Functor f) => Over (Indexed i) (Bazaar' (Indexed i) a) s t a a -> Over p f s t [a] [a]
-ipartsOf' l f s = outs b <$> indexed f (is :: [i]) as where
-  (is,as) = unzip (pins b)
-  b = l sell s
+ipartsOf' l = conjoined
+  (\f s -> let b = inline l sell s                            in outs b <$> f (wins b))
+  (\f s -> let b = inline l sell s; (is, as) = unzip (pins b) in outs b <$> indexed f (is :: [i]) as)
 {-# INLINE ipartsOf' #-}
 
 -- | 'unsafePartsOf' turns a 'Traversal' into a 'Data.Data.Lens.uniplate' (or 'Data.Data.Lens.biplate') family.
@@ -513,9 +559,9 @@ unsafePartsOf l f s = unsafeOuts b <$> f (ins b) where b = l sell s
 
 -- | An indexed version of 'unsafePartsOf' that receives the entire list of indices as its index.
 iunsafePartsOf :: forall i p f s t a b. (Indexable [i] p, Functor f) => Traversing (Indexed i) f s t a b -> Over p f s t [a] [b]
-iunsafePartsOf l f s = unsafeOuts b <$> indexed f (is :: [i]) as where
-  (is,as) = unzip (pins b)
-  b = l sell s
+iunsafePartsOf l = conjoined
+  (\f s -> let b = inline l sell s                           in unsafeOuts b <$> f (wins b))
+  (\f s -> let b = inline l sell s; (is,as) = unzip (pins b) in unsafeOuts b <$> indexed f (is :: [i]) as)
 {-# INLINE iunsafePartsOf #-}
 
 unsafePartsOf' :: ATraversal s t a b -> Lens s t [a] [b]
@@ -523,9 +569,9 @@ unsafePartsOf' l f s = unsafeOuts b <$> f (ins b) where b = l sell s
 {-# INLINE unsafePartsOf' #-}
 
 iunsafePartsOf' :: forall i s t a b. Over (Indexed i) (Bazaar (Indexed i) a b) s t a b -> IndexedLens [i] s t [a] [b]
-iunsafePartsOf' l f s = unsafeOuts b <$> indexed f (is :: [i]) as where
-  (is,as) = unzip (pins b)
-  b = l sell s
+iunsafePartsOf' l = conjoined
+  (\f s -> let b = inline l sell s                            in unsafeOuts b <$> f (wins b))
+  (\f s -> let b = inline l sell s; (is, as) = unzip (pins b) in unsafeOuts b <$> indexed f (is :: [i]) as)
 {-# INLINE iunsafePartsOf' #-}
 
 -- | The one-level version of 'Control.Lens.Plated.contextsOf'. This extracts a list of the immediate children according to a given 'Traversal' as editable contexts.
@@ -544,59 +590,81 @@ iunsafePartsOf' l f s = unsafeOuts b <$> indexed f (is :: [i]) as where
 -- 'holesOf' :: 'IndexedLens'' i s a      -> s -> ['Pretext'' ('Indexed' i) a s]
 -- 'holesOf' :: 'IndexedTraversal'' i s a -> s -> ['Pretext'' ('Indexed' i) a s]
 -- @
-holesOf :: Conjoined p => Over p (Bazaar p a a) s t a a -> s -> [Pretext p a a t]
-holesOf l s = f (pins b) (unsafeOuts b) where
-  b = l sell s
-  f [] _ = []
-  f (wx:xs) g = Pretext (\wxfy -> g . (:Prelude.map extract xs) <$> corep wxfy wx) : f xs (g . (extract wx:))
+holesOf :: forall p s t a. Conjoined p => Over p (Bazaar p a a) s t a a -> s -> [Pretext p a a t]
+holesOf l s = unTagged
+  ( conjoined
+     (Tagged $ let
+        f [] _ = []
+        f (x:xs) g = Pretext (\xfy -> g . (:xs) <$> xfy x) : f xs (g . (x:))
+      in f (ins b) (unsafeOuts b))
+     (Tagged $ let
+        f [] _ = []
+        f (wx:xs) g = Pretext (\wxfy -> g . (:Prelude.map extract xs) <$> corep wxfy wx) : f xs (g . (extract wx:))
+      in f (pins b) (unsafeOuts b))
+    :: Tagged (p a b) [Pretext p a a t]
+  ) where b = l sell s
 {-# INLINE holesOf #-}
 
 -- | This converts a 'Traversal' that you \"know\" will target one or more elements to a 'Lens'. It can
--- also be used to transform a non-empty 'Fold' into a 'Getter' or a non-empty 'MonadicFold' into an
--- 'Action'.
+-- also be used to transform a non-empty 'Fold' into a 'Getter'.
 --
--- The resulting 'Lens', 'Getter', or 'Action' will be partial if the supplied 'Traversal' returns
+-- The resulting 'Lens' or 'Getter' will be partial if the supplied 'Traversal' returns
 -- no results.
 --
+-- >>> [1,2,3] ^. singular _head
+-- 1
+--
+-- >>> [] ^. singular _head
+-- *** Exception: singular: empty traversal
+--
+-- >>> Left 4 ^. singular _Left
+-- 4
+--
+-- >>> [1..10] ^. singular (ix 7)
+-- 8
+--
 -- @
 -- 'singular' :: 'Traversal' s t a a          -> 'Lens' s t a a
 -- 'singular' :: 'Fold' s a                   -> 'Getter' s a
--- 'singular' :: 'MonadicFold' m s a          -> 'Action' m s a
 -- 'singular' :: 'IndexedTraversal' i s t a a -> 'IndexedLens' i s t a a
 -- 'singular' :: 'IndexedFold' i s a          -> 'IndexedGetter' i s a
--- 'singular' :: 'IndexedMonadicFold' i m s a -> 'IndexedAction' i m s a
 -- @
 singular :: (Conjoined p, Functor f)
-         => Over p (BazaarT p f a a) s t a a
+         => Traversing p f s t a a
          -> Over p f s t a a
-singular l pafb s = case pins b of
-  (w:ws) -> unsafeOuts b . (:Prelude.map extract ws) <$> corep pafb w
-  []     -> unsafeOuts b . return                    <$> corep pafb (error "singular: empty traversal")
-  where b = l sell s
+singular l = conjoined
+  (\afb s -> let b = l sell s in case ins b of
+    (w:ws) -> unsafeOuts b . (:ws) <$> afb w
+    []     -> unsafeOuts b . return <$> afb (error "singular: empty traversal"))
+  (\pafb s -> let b = l sell s in case pins b of
+    (w:ws) -> unsafeOuts b . (:Prelude.map extract ws) <$> corep pafb w
+    []     -> unsafeOuts b . return                    <$> corep pafb (error "singular: empty traversal"))
 {-# INLINE singular #-}
 
 -- | This converts a 'Traversal' that you \"know\" will target only one element to a 'Lens'. It can also be
--- used to transform a 'Fold' into a 'Getter' or a 'MonadicFold' into an 'Action'.
+-- used to transform a 'Fold' into a 'Getter'.
 --
--- The resulting 'Lens', 'Getter', or 'Action' will be partial if the 'Traversal' targets nothing
+-- The resulting 'Lens' or 'Getter' will be partial if the 'Traversal' targets nothing
 -- or more than one element.
 --
 -- @
 -- 'unsafeSingular' :: 'Traversal' s t a b          -> 'Lens' s t a b
 -- 'unsafeSingular' :: 'Fold' s a                   -> 'Getter' s a
--- 'unsafeSingular' :: 'MonadicFold' m s a          -> 'Action' m s a
 -- 'unsafeSingular' :: 'IndexedTraversal' i s t a b -> 'IndexedLens' i s t a b
 -- 'unsafeSingular' :: 'IndexedFold' i s a          -> 'IndexedGetter' i s a
--- 'unsafeSingular' :: 'IndexedMonadicFold' i m s a -> 'IndexedAction' i m s a
 -- @
 unsafeSingular :: (Conjoined p, Functor f)
-               => Over p (BazaarT p f a b) s t a b
+               => Traversing p f s t a b
                -> Over p f s t a b
-unsafeSingular l pafb s = case pins b of
-  [w] -> unsafeOuts b . return <$> corep pafb w
-  []  -> error "unsafeSingular: empty traversal"
-  _   -> error "unsafeSingular: traversing multiple results"
-  where b = l sell s
+unsafeSingular l = conjoined
+  (\afb s -> let b = inline l sell s in case ins b of
+    [w] -> unsafeOuts b . return <$> afb w
+    []  -> error "unsafeSingular: empty traversal"
+    _   -> error "unsafeSingular: traversing multiple results")
+  (\pafb s -> let b = inline l sell s in case pins b of
+    [w] -> unsafeOuts b . return <$> corep pafb w
+    []  -> error "unsafeSingular: empty traversal"
+    _   -> error "unsafeSingular: traversing multiple results")
 {-# INLINE unsafeSingular #-}
 
 ------------------------------------------------------------------------------
@@ -607,6 +675,10 @@ ins :: Bizarre (->) w => w a b t -> [a]
 ins = toListOf (coerced bazaar)
 {-# INLINE ins #-}
 
+wins :: (Bizarre p w, Corepresentable p, Comonad (Corep p)) => w a b t -> [a]
+wins = getConst #. bazaar (cotabulate $ \ra -> Const [extract ra])
+{-# INLINE wins #-}
+
 pins :: (Bizarre p w, Corepresentable p) => w a b t -> [Corep p a]
 pins = getConst #. bazaar (cotabulate $ \ra -> Const [ra])
 {-# INLINE pins #-}
@@ -633,9 +705,11 @@ unconsWithDefault _ (x:xs) = (x,xs)
 -- Traversals
 ------------------------------------------------------------------------------
 
--- | Traverse both parts of a tuple with matching types.
+-- | Traverse both parts of a 'Bitraversable' container with matching types.
 --
--- >>> both *~ 10 $ (1,2)
+-- Usually that type will be a pair.
+--
+-- >>> (1,2) & both *~ 10
 -- (10,20)
 --
 -- >>> over both length ("hello","world")
@@ -643,11 +717,22 @@ unconsWithDefault _ (x:xs) = (x,xs)
 --
 -- >>> ("hello","world")^.both
 -- "helloworld"
-both :: Traversal (a,a) (b,b) a b
-both f ~(a,a') = (,) <$> f a <*> f a'
+--
+-- @
+-- 'both' :: 'Traversal' (a, a)       (b, b)       a b
+-- 'both' :: 'Traversal' ('Either' a a) ('Either' b b) a b
+-- @
+both :: Bitraversable r => Traversal (r a a) (r b b) a b
+both f = bitraverse f f
 {-# INLINE both #-}
 
--- | Apply a different 'Traversal' or 'Fold' to each side of a tuple.
+-- | Apply a different 'Traversal' or 'Fold' to each side of a 'Bitraversable' container.
+--
+-- @
+-- 'beside' :: 'Traversal' s t a b                -> 'Traversal' s' t' a b                -> 'Traversal' (r s s') (r t t') a b
+-- 'beside' :: 'IndexedTraversal' i s t a b       -> 'IndexedTraversal' i s' t' a b       -> 'IndexedTraversal' i (r s s') (r t t') a b
+-- 'beside' :: 'IndexPreservingTraversal' s t a b -> 'IndexPreservingTraversal' s' t' a b -> 'IndexPreservingTraversal' (r s s') (r t t') a b
+-- @
 --
 -- @
 -- 'beside' :: 'Traversal' s t a b                -> 'Traversal' s' t' a b                -> 'Traversal' (s,s') (t,t') a b
@@ -678,11 +763,11 @@ both f ~(a,a') = (,) <$> f a <*> f a'
 --
 -- >>> ("hello",["world","!!!"])^..beside id traverse
 -- ["hello","world","!!!"]
-beside :: (Representable q, Applicative (Rep q), Applicative f)
-       => Overloading p q f s t a b
-       -> Overloading p q f s' t' a b
-       -> Overloading p q f (s,s') (t,t') a b
-beside l r f = tabulate $ \ ~(s,s') -> liftA2 (,) <$> rep (l f) s <*> rep (r f) s'
+beside :: (Representable q, Applicative (Rep q), Applicative f, Bitraversable r)
+       => Optical p q f s t a b
+       -> Optical p q f s' t' a b
+       -> Optical p q f (r s s') (r t t') a b
+beside l r f = tabulate $ getCompose #. bitraverse (Compose #. rep (l f)) (Compose #. rep (r f))
 {-# INLINE beside #-}
 
 -- | Visit the first /n/ targets of a 'Traversal', 'Fold', 'Getter' or 'Lens'.
@@ -703,20 +788,18 @@ beside l r f = tabulate $ \ ~(s,s') -> liftA2 (,) <$> rep (l f) s <*> rep (r f)
 -- 'taking' :: 'Int' -> 'Prism'' s a                       -> 'Traversal'' s a
 -- 'taking' :: 'Int' -> 'Getter' s a                       -> 'Fold' s a
 -- 'taking' :: 'Int' -> 'Fold' s a                         -> 'Fold' s a
--- 'taking' :: 'Int' -> 'Action' m s a                     -> 'MonadicFold' m s a
--- 'taking' :: 'Int' -> 'MonadicFold' m s a                -> 'MonadicFold' m s a
 -- 'taking' :: 'Int' -> 'IndexedTraversal'' i s a          -> 'IndexedTraversal'' i s a
 -- 'taking' :: 'Int' -> 'IndexedLens'' i s a               -> 'IndexedTraversal'' i s a
 -- 'taking' :: 'Int' -> 'IndexedGetter' i s a              -> 'IndexedFold' i s a
 -- 'taking' :: 'Int' -> 'IndexedFold' i s a                -> 'IndexedFold' i s a
--- 'taking' :: 'Int' -> 'IndexedAction' i m s a            -> 'IndexedMonadicFold' i m s a
--- 'taking' :: 'Int' -> 'IndexedMonadicFold' i m s a       -> 'IndexedMonadicFold' i m s a
 -- @
 taking :: (Conjoined p, Applicative f)
         => Int
-       -> Over p (BazaarT p f a a) s t a a
+       -> Traversing p f s t a a
        -> Over p f s t a a
-taking n l pafb s = outs b <$> traverse (corep pafb) (take n $ pins b) where b = l sell s
+taking n l = conjoined
+  (\ afb s  -> let b = inline l sell s in outs b <$> traverse afb          (take n $ ins b))
+  (\ pafb s -> let b = inline l sell s in outs b <$> traverse (corep pafb) (take n $ pins b))
 {-# INLINE taking #-}
 
 -- | Visit all but the first /n/ targets of a 'Traversal', 'Fold', 'Getter' or 'Lens'.
@@ -736,14 +819,10 @@ taking n l pafb s = outs b <$> traverse (corep pafb) (take n $ pins b) where b =
 -- 'dropping' :: 'Int' -> 'Prism'' s a                       -> 'Traversal'' s a
 -- 'dropping' :: 'Int' -> 'Getter' s a                       -> 'Fold' s a
 -- 'dropping' :: 'Int' -> 'Fold' s a                         -> 'Fold' s a
--- 'dropping' :: 'Int' -> 'Action' m s a                     -> 'MonadicFold' m s a
--- 'dropping' :: 'Int' -> 'MonadicFold' m s a                -> 'MonadicFold' m s a
 -- 'dropping' :: 'Int' -> 'IndexedTraversal'' i s a          -> 'IndexedTraversal'' i s a
 -- 'dropping' :: 'Int' -> 'IndexedLens'' i s a               -> 'IndexedTraversal'' i s a
 -- 'dropping' :: 'Int' -> 'IndexedGetter' i s a              -> 'IndexedFold' i s a
 -- 'dropping' :: 'Int' -> 'IndexedFold' i s a                -> 'IndexedFold' i s a
--- 'dropping' :: 'Int' -> 'IndexedAction' i m s a            -> 'IndexedMonadicFold' i m s a
--- 'dropping' :: 'Int' -> 'IndexedMonadicFold' i m s a       -> 'IndexedMonadicFold' i m s a
 -- @
 dropping :: (Conjoined p, Applicative f) => Int -> Over p (Indexing f) s t a a -> Over p f s t a a
 dropping n l pafb s = snd $ runIndexing (l paifb s) 0 where
@@ -790,6 +869,22 @@ cloneIndexedTraversal :: AnIndexedTraversal i s t a b -> IndexedTraversal i s t
 cloneIndexedTraversal l f = bazaar (Indexed (indexed f)) . l sell
 {-# INLINE cloneIndexedTraversal #-}
 
+-- | A 'Traversal1' is completely characterized by its behavior on a 'Bazaar1'.
+cloneTraversal1 :: ATraversal1 s t a b -> Traversal1 s t a b
+cloneTraversal1 l f = bazaar1 f . l sell
+{-# INLINE cloneTraversal1 #-}
+
+-- | Clone a 'Traversal1' yielding an 'IndexPreservingTraversal1' that passes through
+-- whatever index it is composed with.
+cloneIndexPreservingTraversal1 :: ATraversal1 s t a b -> IndexPreservingTraversal1 s t a b
+cloneIndexPreservingTraversal1 l pafb = cotabulate $ \ws -> runBazaar1 (l sell (extract ws)) $ \a -> corep pafb (a <$ ws)
+{-# INLINE cloneIndexPreservingTraversal1 #-}
+
+-- | Clone an 'IndexedTraversal1' yielding an 'IndexedTraversal1' with the same index.
+cloneIndexedTraversal1 :: AnIndexedTraversal1 i s t a b -> IndexedTraversal1 i s t a b
+cloneIndexedTraversal1 l f = bazaar1 (Indexed (indexed f)) . l sell
+{-# INLINE cloneIndexedTraversal1 #-}
+
 ------------------------------------------------------------------------------
 -- Indexed Traversals
 ------------------------------------------------------------------------------
@@ -805,8 +900,9 @@ cloneIndexedTraversal l f = bazaar (Indexed (indexed f)) . l sell
 -- @
 --
 -- @
--- 'itraverseOf' :: 'Functor' f     => 'IndexedLens' i s t a b      -> (i -> a -> f b) -> s -> f t
--- 'itraverseOf' :: 'Applicative' f => 'IndexedTraversal' i s t a b -> (i -> a -> f b) -> s -> f t
+-- 'itraverseOf' :: 'Functor' f     => 'IndexedLens' i s t a b       -> (i -> a -> f b) -> s -> f t
+-- 'itraverseOf' :: 'Applicative' f => 'IndexedTraversal' i s t a b  -> (i -> a -> f b) -> s -> f t
+-- 'itraverseOf' :: 'Apply' f       => 'IndexedTraversal1' i s t a b -> (i -> a -> f b) -> s -> f t
 -- @
 itraverseOf :: (Indexed i a (f b) -> s -> f t) -> (i -> a -> f b) -> s -> f t
 itraverseOf l = l .# Indexed
@@ -820,8 +916,9 @@ itraverseOf l = l .# Indexed
 -- @
 --
 -- @
--- 'iforOf' :: 'Functor' f => 'IndexedLens' i s t a b      -> s -> (i -> a -> f b) -> f t
--- 'iforOf' :: 'Applicative' f => 'IndexedTraversal' i s t a b -> s -> (i -> a -> f b) -> f t
+-- 'iforOf' :: 'Functor' f     => 'IndexedLens' i s t a b       -> s -> (i -> a -> f b) -> f t
+-- 'iforOf' :: 'Applicative' f => 'IndexedTraversal' i s t a b  -> s -> (i -> a -> f b) -> f t
+-- 'iforOf' :: 'Apply' f       => 'IndexedTraversal1' i s t a b -> s -> (i -> a -> f b) -> f t
 -- @
 iforOf :: (Indexed i a (f b) -> s -> f t) -> s -> (i -> a -> f b) -> f t
 iforOf = flip . itraverseOf
@@ -838,8 +935,9 @@ iforOf = flip . itraverseOf
 -- @
 --
 -- @
--- 'imapMOf' :: 'Monad' m => 'IndexedLens'      i s t a b -> (i -> a -> m b) -> s -> m t
--- 'imapMOf' :: 'Monad' m => 'IndexedTraversal' i s t a b -> (i -> a -> m b) -> s -> m t
+-- 'imapMOf' :: 'Monad' m => 'IndexedLens'       i s t a b -> (i -> a -> m b) -> s -> m t
+-- 'imapMOf' :: 'Monad' m => 'IndexedTraversal'  i s t a b -> (i -> a -> m b) -> s -> m t
+-- 'imapMOf' :: 'Bind'  m => 'IndexedTraversal1' i s t a b -> (i -> a -> m b) -> s -> m t
 -- @
 imapMOf :: (Indexed i a (WrappedMonad m b) -> s -> WrappedMonad m t) -> (i -> a -> m b) -> s -> m t
 imapMOf l = mapMOf l .# Indexed
@@ -903,6 +1001,11 @@ traversed :: Traversable f => IndexedTraversal Int (f a) (f b) a b
 traversed = conjoined traverse (indexing traverse)
 {-# INLINE traversed #-}
 
+-- | Traverse any 'Traversable1' container. This is an 'IndexedTraversal1' that is indexed by ordinal position.
+traversed1 :: Traversable1 f => IndexedTraversal1 Int (f a) (f b) a b
+traversed1 = conjoined traverse1 (indexing traverse1)
+{-# INLINE traversed1 #-}
+
 -- | Traverse any 'Traversable' container. This is an 'IndexedTraversal' that is indexed by ordinal position.
 traversed64 :: Traversable f => IndexedTraversal Int64 (f a) (f b) a b
 traversed64 = conjoined traverse (indexing64 traverse)
@@ -1033,8 +1136,65 @@ elements = elementsOf traverse
 --
 -- >>> failover _Right (*2) (Right 4) :: Maybe (Either Int Int)
 -- Just (Right 8)
-failover :: MonadPlus m => LensLike ((,) Any) s t a b -> (a -> b) -> s -> m t
-failover l f s = case l ((,) (Any True) . f) s of
-  (Any True, t)  -> return t
-  (Any False, _) -> mzero
+--
+-- @
+-- 'failover' :: Alternative m => Traversal s t a b -> (a -> b) -> s -> m t
+-- @
+failover :: (Profunctor p, Alternative m) => Over p ((,) Any) s t a b -> p a b -> s -> m t
+failover l pafb s = case l ((,) (Any True) `rmap` pafb) s of
+  (Any True, t)  -> pure t
+  (Any False, _) -> Applicative.empty
 {-# INLINE failover #-}
+
+-- | Try to map a function which uses the index over this 'IndexedTraversal', failing if the 'IndexedTraversal' has no targets.
+--
+-- @
+-- 'ifailover' :: Alternative m => IndexedTraversal i s t a b -> (i -> a -> b) -> s -> m t
+-- @
+ifailover :: Alternative m => Over (Indexed i) ((,) Any) s t a b -> (i -> a -> b) -> s -> m t
+ifailover l f = failover l (Indexed f)
+{-# INLINE ifailover #-}
+
+-- | Try the first 'Traversal' (or 'Fold'), falling back on the second 'Traversal' (or 'Fold') if it returns no entries.
+--
+-- This is only a valid 'Traversal' if the second 'Traversal' is disjoint from the result of the first or returns
+-- exactly the same results. These conditions are trivially met when given a 'Lens', 'Iso', 'Getter', 'Prism' or \"affine\" Traversal -- one that
+-- has 0 or 1 target.
+--
+-- Mutatis mutandis for 'Fold'.
+--
+-- @
+-- 'failing' :: 'Traversal' s t a b -> 'Traversal' s t a b -> 'Traversal' s t a b
+-- 'failing' :: 'Prism' s t a b     -> 'Prism' s t a b     -> 'Traversal' s t a b
+-- 'failing' :: 'Fold' s a          -> 'Fold' s a          -> 'Fold' s a
+-- @
+--
+-- These cases are also supported, trivially, but are boring, because the left hand side always succeeds.
+--
+-- @
+-- 'failing' :: 'Lens' s t a b      -> 'Traversal' s t a b -> 'Traversal' s t a b
+-- 'failing' :: 'Iso' s t a b       -> 'Traversal' s t a b -> 'Traversal' s t a b
+-- 'failing' :: 'Equality' s t a b  -> 'Traversal' s t a b -> 'Traversal' s t a b
+-- 'failing' :: 'Getter' s a        -> 'Fold' s a          -> 'Fold' s a
+-- @
+--
+-- If both of the inputs are indexed, the result is also indexed, so you can apply this to a pair of indexed
+-- traversals or indexed folds, obtaining an indexed traversal or indexed fold.
+--
+-- @
+-- 'failing' :: 'IndexedTraversal' i s t a b -> 'IndexedTraversal' i s t a b -> 'IndexedTraversal' i s t a b
+-- 'failing' :: 'IndexedFold' i s a          -> 'IndexedFold' i s a          -> 'IndexedFold' i s a
+-- @
+--
+-- These cases are also supported, trivially, but are boring, because the left hand side always succeeds.
+--
+-- @
+-- 'failing' :: 'IndexedLens' i s t a b      -> 'IndexedTraversal' i s t a b -> 'IndexedTraversal' i s t a b
+-- 'failing' :: 'IndexedGetter' i s a        -> 'IndexedGetter' i s a        -> 'IndexedFold' i s a
+-- @
+failing :: (Conjoined p, Applicative f) => Traversing p f s t a b -> Traversing p f s t a b -> Over p f s t a b
+failing l r pafb s = case pins b of
+  [] -> runBazaarT (r sell s) pafb
+  xs -> unsafeOuts b <$> traverse (corep pafb) xs
+  where b = l sell s
+infixl 5 `failing`
