@@ -9,7 +9,7 @@
 -------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Type
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -22,34 +22,42 @@
 module Control.Lens.Type
   (
   -- * Other
-    Equality, Equality'
+    Equality, Equality', As
   , Iso, Iso'
   , Prism , Prism'
   -- * Lenses, Folds and Traversals
   , Lens, Lens'
   , Traversal, Traversal'
+  , Traversal1, Traversal1'
   , Setter, Setter'
   , Getter, Fold
-  , Action, MonadicFold
+  , Fold1
+  , Action, MonadicFold, RelevantMonadicFold
   -- * Indexed
   , IndexedLens, IndexedLens'
   , IndexedTraversal, IndexedTraversal'
+  , IndexedTraversal1, IndexedTraversal1'
   , IndexedSetter, IndexedSetter'
   , IndexedGetter, IndexedFold
+  , IndexedFold1
   , IndexedAction, IndexedMonadicFold
+  , IndexedRelevantMonadicFold
   -- * Index-Preserving
   , IndexPreservingLens, IndexPreservingLens'
   , IndexPreservingTraversal, IndexPreservingTraversal'
+  , IndexPreservingTraversal1, IndexPreservingTraversal1'
   , IndexPreservingSetter, IndexPreservingSetter'
   , IndexPreservingGetter, IndexPreservingFold
+  , IndexPreservingFold1
   , IndexPreservingAction, IndexPreservingMonadicFold
+  , IndexPreservingRelevantMonadicFold
   -- * Common
   , Simple
   , LensLike, LensLike'
   , Over, Over'
   , IndexedLensLike, IndexedLensLike'
-  , Overloading, Overloading'
-  , Overloaded, Overloaded'
+  , Optical, Optical'
+  , Optic, Optic'
   ) where
 
 import Control.Applicative
@@ -57,6 +65,7 @@ import Control.Lens.Internal.Action
 import Control.Lens.Internal.Setter
 import Control.Lens.Internal.Indexed
 import Data.Functor.Contravariant
+import Data.Functor.Apply
 import Data.Profunctor
 
 -- $setup
@@ -85,19 +94,19 @@ import Data.Profunctor
 -- 1) You get back what you put in:
 --
 -- @
--- 'Control.Lens.Getter.view' l ('Control.Lens.Setter.set' l b a)  ≡ b
+-- 'Control.Lens.Getter.view' l ('Control.Lens.Setter.set' l v s)  ≡ v
 -- @
 --
 -- 2) Putting back what you got doesn't change anything:
 --
 -- @
--- 'Control.Lens.Setter.set' l ('Control.Lens.Getter.view' l a) a  ≡ a
+-- 'Control.Lens.Setter.set' l ('Control.Lens.Getter.view' l s) s  ≡ s
 -- @
 --
 -- 3) Setting twice is the same as setting once:
 --
 -- @
--- 'Control.Lens.Setter.set' l c ('Control.Lens.Setter.set' l b a) ≡ 'Control.Lens.Setter.set' l c a
+-- 'Control.Lens.Setter.set' l v' ('Control.Lens.Setter.set' l v s) ≡ 'Control.Lens.Setter.set' l v' s
 -- @
 --
 -- These laws are strong enough that the 4 type parameters of a 'Lens' cannot
@@ -105,8 +114,19 @@ import Data.Profunctor
 -- it a Lens Family?\" section of
 -- <http://comonad.com/reader/2012/mirrored-lenses/>.
 --
--- Every 'Lens' can be used directly as a 'Control.Lens.Setter.Setter' or
--- 'Traversal'.
+-- There are some emergent properties of these laws:
+--
+-- 1) @'Control.Lens.Setter.set' l s@ must be injective for every @s@ This is a consequence of law #1
+--
+-- 2) @'Control.Lens.Setter.set' l@ must be surjective, because of law #2, which indicates that it is possible to obtain any 'v' from some 's' such that @'Control.Lens.Setter.set' s v = s@
+--
+-- 3) Given just the first two laws you can prove a weaker form of law #3 where the values @v@ that you are setting match:
+--
+-- @
+-- 'Control.Lens.Setter.set' l v ('Control.Lens.Setter.set' l v s) ≡ 'Control.Lens.Setter.set' l v s
+-- @
+--
+-- Every 'Lens' can be used directly as a 'Control.Lens.Setter.Setter' or 'Traversal'.
 --
 -- You can also use a 'Lens' for 'Control.Lens.Getter.Getting' as if it were a
 -- 'Fold' or 'Getter'.
@@ -183,6 +203,9 @@ type Traversal s t a b = forall f. Applicative f => (a -> f b) -> s -> f t
 -- @
 type Traversal' s a = Traversal s s a a
 
+type Traversal1 s t a b = forall f. Apply f => (a -> f b) -> s -> f t
+type Traversal1' s a = Traversal1 s s a a
+
 -- | Every 'IndexedTraversal' is a valid 'Control.Lens.Traversal.Traversal' or
 -- 'Control.Lens.Fold.IndexedFold'.
 --
@@ -197,6 +220,9 @@ type IndexedTraversal i s t a b = forall p f. (Indexable i p, Applicative f) =>
 -- @
 type IndexedTraversal' i s a = IndexedTraversal i s s a a
 
+type IndexedTraversal1 i s t a b = forall p f. (Indexable i p, Apply f) => p a (f b) -> s -> f t
+type IndexedTraversal1' i s a = IndexedTraversal1 i s s a a
+
 -- | An 'IndexPreservingLens' leaves any index it is composed with alone.
 type IndexPreservingTraversal s t a b = forall p f. (Conjoined p, Applicative f) => p a (f b) -> p s (f t)
 
@@ -205,6 +231,9 @@ type IndexPreservingTraversal s t a b = forall p f. (Conjoined p, Applicative f)
 -- @
 type IndexPreservingTraversal' s a = IndexPreservingTraversal s s a a
 
+type IndexPreservingTraversal1 s t a b = forall p f. (Conjoined p, Apply f) => p a (f b) -> p s (f t)
+type IndexPreservingTraversal1' s a = IndexPreservingTraversal1 s s a a
+
 ------------------------------------------------------------------------------
 -- Setters
 ------------------------------------------------------------------------------
@@ -301,7 +330,7 @@ type Iso' s a = Iso s s a a
 -- Prism Internals
 ------------------------------------------------------------------------------
 
--- | A 'Prism' @l@ is a 0-or-1 target 'Traversal' that can also be turned
+-- | A 'Prism' @l@ is a 'Traversal' that can also be turned
 -- around with 'Control.Lens.Review.re' to obtain a 'Getter' in the
 -- opposite direction.
 --
@@ -396,6 +425,10 @@ type Equality s t a b = forall p (f :: * -> *). p a (f b) -> p s (f t)
 -- | A 'Simple' 'Equality'.
 type Equality' s a = Equality s s a a
 
+-- | Composable `asTypeOf`. Useful for constraining excess
+-- polymorphism, @foo . (id :: As Int) . bar@.
+type As a = Equality' a a
+
 -------------------------------------------------------------------------------
 -- Getters
 -------------------------------------------------------------------------------
@@ -405,7 +438,7 @@ type Equality' s a = Equality s s a a
 --
 -- Unlike a 'Lens' a 'Getter' is read-only. Since a 'Getter'
 -- cannot be used to write back there are no 'Lens' laws that can be applied to
--- it. In fact, it is isomorphic to an arbitrary function from @(a -> s)@.
+-- it. In fact, it is isomorphic to an arbitrary function from @(s -> a)@.
 --
 -- Moreover, a 'Getter' can be used directly as a 'Control.Lens.Fold.Fold',
 -- since it just ignores the 'Applicative'.
@@ -444,6 +477,11 @@ type IndexedFold i s a = forall p f.  (Indexable i p, Contravariant f, Applicati
 -- 'IndexedFold', or 'IndexedLens' yields an 'IndexedFold' respectively.
 type IndexPreservingFold s a = forall p f. (Conjoined p, Contravariant f, Applicative f) => p a (f a) -> p s (f s)
 
+-- | A relevant Fold (aka 'Fold1') has one or more targets.
+type Fold1 s a = forall f. (Contravariant f, Apply f) => (a -> f a) -> s -> f s
+type IndexedFold1 i s a = forall p f.  (Indexable i p, Contravariant f, Apply f) => p a (f a) -> s -> f s
+type IndexPreservingFold1 s a = forall p f. (Conjoined p, Contravariant f, Apply f) => p a (f a) -> p s (f s)
+
 -------------------------------------------------------------------------------
 -- Actions
 -------------------------------------------------------------------------------
@@ -476,6 +514,7 @@ type IndexPreservingAction m s a = forall p f r. (Conjoined p, Effective m r f)
 --
 -- You can compose a 'MonadicFold' with another 'MonadicFold' using ('Prelude..') from the @Prelude@.
 type MonadicFold m s a = forall f r. (Effective m r f, Applicative f) => (a -> f a) -> s -> f s
+type RelevantMonadicFold m s a = forall f r. (Effective m r f, Apply f) => (a -> f a) -> s -> f s
 
 -- | An 'IndexedMonadicFold' is an 'IndexedFold' enriched with access to a 'Monad' for side-effects.
 --
@@ -483,10 +522,12 @@ type MonadicFold m s a = forall f r. (Effective m r f, Applicative f) => (a -> f
 --
 -- You can compose an 'IndexedMonadicFold' with another 'IndexedMonadicFold' using ('Prelude..') from the @Prelude@.
 type IndexedMonadicFold i m s a = forall p f r. (Indexable i p, Effective m r f, Applicative f) => p a (f a) -> s -> f s
+type IndexedRelevantMonadicFold i m s a = forall p f r. (Indexable i p, Effective m r f, Apply f) => p a (f a) -> s -> f s
 
 -- | An 'IndexPreservingFold' can be used as a 'Fold', but when composed with an 'IndexedTraversal',
 -- 'IndexedFold', or 'IndexedLens' yields an 'IndexedFold' respectively.
 type IndexPreservingMonadicFold m s a = forall p f r. (Conjoined p, Effective m r f, Applicative f) => p a (f a) -> p s (f s)
+type IndexPreservingRelevantMonadicFold m s a = forall p f r. (Conjoined p, Effective m r f, Apply f) => p a (f a) -> p s (f s)
 
 -------------------------------------------------------------------------------
 -- Simple Overloading
@@ -507,25 +548,41 @@ type IndexPreservingMonadicFold m s a = forall p f r. (Conjoined p, Effective m
 -- This is commonly abbreviated as a \"prime\" marker, /e.g./ 'Lens'' = 'Simple' 'Lens'.
 type Simple f s a = f s s a a
 
--- | @
--- type 'LensLike' f s t a b = 'Overloading' (->) (->) f s t a b
+-------------------------------------------------------------------------------
+-- Optics
+-------------------------------------------------------------------------------
+
+-- | A valid 'Optic' @l@ should satisfy the laws:
+--
 -- @
-type Overloading p q f s t a b = p a (f b) -> q s (f t)
+-- l 'pure' ≡ 'pure'
+-- l ('Procompose' f g) = 'Procompose' (l f) (l g)
+-- @
+--
+-- This gives rise to the laws for 'Equality', 'Iso', 'Prism', 'Lens',
+-- 'Traversal', 'Traversal1', 'Setter', 'Fold', 'Fold1', and 'Getter' as well
+-- along with their index-preserving variants.
+--
+-- @
+-- type 'LensLike' f s t a b = 'Optic' (->) f s t a b
+-- @
+type Optic p f s t a b = p a (f b) -> p s (f t)
 
 -- | @
--- type 'Overloading'' p q f s a = 'Simple' ('Overloading' p q f) s a
+-- type 'Optic'' p q f s a = 'Simple' ('Optic' p q f) s a
 -- @
-type Overloading' p q f s a = Overloading p q f s s a a
+type Optic' p f s a = Optic p f s s a a
 
 -- | @
--- type 'LensLike' f s t a b = 'Overloaded' (->) f s t a b
+-- type 'LensLike' f s t a b = 'Optical' (->) (->) f s t a b
 -- @
-type Overloaded p f s t a b = p a (f b) -> p s (f t)
+type Optical p q f s t a b = p a (f b) -> q s (f t)
 
 -- | @
--- type 'Overloaded'' p q f s a = 'Simple' ('Overloaded' p q f) s a
+-- type 'Optical'' p q f s a = 'Simple' ('Optical' p q f) s a
 -- @
-type Overloaded' p f s a = Overloaded p f s s a a
+type Optical' p q f s a = Optical p q f s s a a
+
 
 -- | Many combinators that accept a 'Lens' can also accept a
 -- 'Traversal' in limited situations.
