@@ -3,7 +3,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Bits.Lens
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -11,8 +11,8 @@
 --
 ----------------------------------------------------------------------------
 module Data.Bits.Lens
-  ( (.|.~), (.&.~), (<.|.~), (<.&.~)
-  , (.|.=), (.&.=), (<.|.=), (<.&.=)
+  ( (.|.~), (.&.~), (<.|.~), (<.&.~), (<<.|.~), (<<.&.~)
+  , (.|.=), (.&.=), (<.|.=), (<.&.=), (<<.|.=), (<<.&.=)
   , bitAt
   , bits
   , byteAt
@@ -28,8 +28,8 @@ import Data.Word
 -- >>> :set -XNoOverloadedStrings
 -- >>> import Data.Word
 
-infixr 4 .|.~, .&.~, <.|.~, <.&.~
-infix 4 .|.=, .&.=, <.|.=, <.&.=
+infixr 4 .|.~, .&.~, <.|.~, <.&.~, <<.|.~, <<.&.~
+infix 4 .|.=, .&.=, <.|.=, <.&.=, <<.|.=, <<.&.=
 
 -- | Bitwise '.|.' the target(s) of a 'Lens' or 'Setter'.
 --
@@ -149,6 +149,22 @@ l <.&.= b = l <%= (.&. b)
 l <.|.= b = l <%= (.|. b)
 {-# INLINE (<.|.=) #-}
 
+(<<.&.~) :: Bits a => Optical' (->) q ((,)a) s a -> a -> q s (a, s)
+l <<.&.~ b = l $ \a -> (a, a .&. b)
+{-# INLINE (<<.&.~) #-}
+
+(<<.|.~) :: Bits a => Optical' (->) q ((,)a) s a -> a -> q s (a, s)
+l <<.|.~ b = l $ \a -> (a, a .|. b)
+{-# INLINE (<<.|.~) #-}
+
+(<<.&.=) :: (MonadState s m, Bits a) => LensLike' ((,) a) s a -> a -> m a
+l <<.&.= b = l %%= \a -> (a, a .&. b)
+{-# INLINE (<<.&.=) #-}
+
+(<<.|.=) :: (MonadState s m, Bits a) => LensLike' ((,) a) s a -> a -> m a
+l <<.|.= b = l %%= \a -> (a, a .|. b)
+{-# INLINE (<<.|.=) #-}
+
 -- | This 'Lens' can be used to access the value of the nth bit in a number.
 --
 -- @'bitAt' n@ is only a legal 'Lens' into @b@ if @0 '<=' n '<' 'bitSize' ('undefined' :: b)@.
