@@ -3,7 +3,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Sequence.Lens
--- Copyright   :  (C) 2012-13 Edward Kmett
+-- Copyright   :  (C) 2012-14 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -13,6 +13,7 @@
 module Data.Sequence.Lens
   ( viewL, viewR
   , sliced, slicedTo, slicedFrom
+  , seqOf
   ) where
 
 import Control.Applicative
@@ -115,3 +116,22 @@ sliced i j f s = case Seq.splitAt i s of
   (l,mr) -> case Seq.splitAt (j-i) mr of
      (m, r) -> itraverse (indexed f . (+i)) m <&> \n -> l >< n >< r
 {-# INLINE sliced #-}
+
+-- | Construct a 'Seq' from a 'Getter', 'Control.Lens.Fold.Fold', 'Control.Lens.Traversal.Traversal', 'Control.Lens.Lens.Lens' or 'Control.Lens.Iso.Iso'.
+--
+-- >>> seqOf folded ["hello","world"]
+-- fromList ["hello","world"]
+--
+-- >>> seqOf (folded._2) [("hello",1),("world",2),("!!!",3)]
+-- fromList [1,2,3]
+--
+-- @
+-- 'seqOf' :: 'Getter' s a     -> s -> 'Seq' a
+-- 'seqOf' :: 'Fold' s a       -> s -> 'Seq' a
+-- 'seqOf' :: 'Iso'' s a       -> s -> 'Seq' a
+-- 'seqOf' :: 'Lens'' s a      -> s -> 'Seq' a
+-- 'seqOf' :: 'Traversal'' s a -> s -> 'Seq' a
+-- @
+seqOf :: Getting (Seq a) s a -> s -> Seq a
+seqOf l = views l Seq.singleton
+{-# INLINE seqOf #-}
