@@ -7,7 +7,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Text.Lazy.Lens
--- Copyright   :  (C) 2012-2013 Edward Kmett
+-- Copyright   :  (C) 2012-2014 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -18,11 +18,14 @@ module Data.Text.Lazy.Lens
   ( packed, unpacked
   , text
   , builder
+  , utf8
   ) where
 
 import Control.Lens
-import Data.Text.Lazy
+import Data.ByteString.Lazy as ByteString
+import Data.Text.Lazy as Text
 import Data.Text.Lazy.Builder
+import Data.Text.Lazy.Encoding
 
 -- $setup
 -- >>> :set -XOverloadedStrings
@@ -38,7 +41,7 @@ import Data.Text.Lazy.Builder
 -- 'packed' ≡ 'from' 'unpacked'
 -- @
 packed :: Iso' String Text
-packed = iso pack unpack
+packed = iso Text.pack Text.unpack
 {-# INLINE packed #-}
 
 -- | This isomorphism can be used to 'unpack' (or 'pack') lazy 'Text'.
@@ -57,7 +60,7 @@ packed = iso pack unpack
 -- 'unpacked' ≡ 'from' 'packed'
 -- @
 unpacked :: Iso' Text String
-unpacked = iso unpack pack
+unpacked = iso Text.unpack Text.pack
 {-# INLINE unpacked #-}
 
 -- | Convert between lazy 'Text' and 'Builder' .
@@ -84,6 +87,20 @@ builder = iso fromLazyText toLazyText
 -- @
 -- 'text' ≡ 'each'
 -- @
+--
+-- Note that when just using this as a 'Setter', @'setting' 'Data.Text.Lazy.map'@
+-- can be more efficient.
 text :: IndexedTraversal' Int Text Char
 text = unpacked . traversed
 {-# INLINE text #-}
+
+-- | Encode/Decode a lazy 'Text' to/from lazy 'ByteString', via UTF-8.
+--
+-- Note: This function does not decode lazily, as it must consume the entire
+-- input before deciding whether or not it fails.
+--
+-- >>> ByteString.unpack (utf8 # "☃")
+-- [226,152,131]
+utf8 :: Prism' ByteString Text
+utf8 = prism' encodeUtf8 (preview _Right . decodeUtf8')
+{-# INLINE utf8 #-}
